/********************************** <lalVerbatim file="COHConditionDataCV">
Author: Julien Sylvestre
$Id: ConditionData.c,v 1.4 2002/12/02 17:42:31 julien Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <ConditionData.h>
#include <InitSearch.h>
#include <strings.h>
#include <lal/AVFactories.h>
#include <math.h>


NRCSID( CONDITIONDATAC , "power $Id: ConditionData.c,v 1.4 2002/12/02 17:42:31 julien Exp $" );


#ifndef COHERE_DEBUG
void
LALConditionData(
    LALStatus             *status,
    LALSearchInput        *inout,
    void                  *searchParams,
    LALMPIParams          *mpiParams
                )
{
  CohereSearchParams *params;

  INITSTATUS (status, "LALConditionData", CONDITIONDATAC);
  ATTATCHSTATUSPTR (status);

  params = (CohereSearchParams *) searchParams;

  ASSERT (searchParams, status, CONDITIONDATAH_ENULL, CONDITIONDATAH_MSGENULL);

  /* do nothing if searchMaster */
  if(!(params->searchMaster)) {

    UINT4 si;
    COMPLEX16Vector **Poles, **Zeros;
    COMPLEX16 *Gain;
    BOOLEAN doFilters = 1;

    Gain = (COMPLEX16 *)LALCalloc(params->NIFO, sizeof(COMPLEX16));
    if(!Gain) {
      ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
    }

    Poles = (COMPLEX16Vector **)LALCalloc(params->NIFO, sizeof(COMPLEX16Vector *));
    if(!Poles) {
      ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
    }
    
    Zeros = (COMPLEX16Vector **)LALCalloc(params->NIFO, sizeof(COMPLEX16Vector *));
    if(!Zeros) {
      ABORT (status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC);
    }


    for(si=0;si<inout->numberSequences;si++) {

      /* Correlation Matrix */
      if(strstr(inout->sequences[si].comment, INPUTNAME_CMAT)) {
	UINT4 ifo_ind;
	char *ifo_name;
	COHIFO IFO;

	/* which IFO ? */
	ifo_name = inout->sequences[si].comment + strlen(INPUTNAME_CMAT);
	if(strstr(ifo_name,"lho2k")) {
	  IFO = LHO2K;
	} else if(strstr(ifo_name,"lho4k")) {
	  IFO = LHO4K;
	} else if(strstr(ifo_name,"llo4k")) {
	  IFO = LLO4K;
	} else if(strstr(ifo_name,"virgo")) {
	  IFO = VIRGO;
	} else if(strstr(ifo_name,"tama")) {
	  IFO = TAMA;
	} else if(strstr(ifo_name,"geo")) {
	  IFO = GEO;
	} else {
	  ABORT ( status, CONDITIONDATAH_ECMAT, CONDITIONDATAH_MSGECMAT );
	}

	for(ifo_ind=0; ifo_ind<params->NIFO; ifo_ind++) {
	  if(params->IFOIND[ifo_ind] == IFO) {break;}
	}

	if(ifo_ind == params->NIFO) {
	  ABORT ( status, CONDITIONDATAH_ECMAT, CONDITIONDATAH_MSGECMAT );
	}

	/* cast */
	switch(inout->sequences[si].type) {

	case real_8:
	  params->IFOInfo.CMat[ifo_ind][ifo_ind] = inout->sequences[si].data.real8[0];
	  break;

#define TFCCAST(TYPE__,PTR__) \
    case TYPE__ : \
	params->IFOInfo.CMat[ifo_ind][ifo_ind] = (REAL8)inout->sequences[si].data.PTR__[0]; \
      break

	  TFCCAST(boolean_lu,boolean);
	  TFCCAST(char_s,chars);
	  TFCCAST(char_u,charu);
	  TFCCAST(int_2s,int2s);
	  TFCCAST(int_2u,int2u);
	  TFCCAST(int_4s,int4s);
	  TFCCAST(int_4u,int4u);
	  TFCCAST(int_8s,int8s);
	  TFCCAST(int_8u,int8u);
	  TFCCAST(real_4,real4);

#undef TFCCAST

	default:
	  ABORT ( status, CONDITIONDATAH_EDTYP, CONDITIONDATAH_MSGEDTYP );
	}
      }

      /* IFO DATA */
      else if(strstr(inout->sequences[si].comment, INPUTNAME_IFODATA)) {

	UINT4 i, ifo_ind;
	char *ifo_name;
	COHIFO IFO;
	REAL4 *dmro;

	/* which IFO ? */
	ifo_name = inout->sequences[si].comment + strlen(INPUTNAME_IFODATA);
	if(strstr(ifo_name,"lho2k")) {
	  IFO = LHO2K;
	} else if(strstr(ifo_name,"lho4k")) {
	  IFO = LHO4K;
	} else if(strstr(ifo_name,"llo4k")) {
	  IFO = LLO4K;
	} else if(strstr(ifo_name,"virgo")) {
	  IFO = VIRGO;
	} else if(strstr(ifo_name,"tama")) {
	  IFO = TAMA;
	} else if(strstr(ifo_name,"geo")) {
	  IFO = GEO;
	} else {
	  ABORT ( status, CONDITIONDATAH_EIFO, CONDITIONDATAH_MSGEIFO );
	}

	for(ifo_ind=0; ifo_ind<params->NIFO; ifo_ind++) {
	  if(params->IFOIND[ifo_ind] == IFO) {break;}
	}

	if(ifo_ind == params->NIFO) {
	  ABORT ( status, CONDITIONDATAH_ENIFO, CONDITIONDATAH_MSGENIFO );
	}

	if(params->NData != inout->sequences[si].dimensions[0]) {
	  ABORT (status, CONDITIONDATAH_ENDAT, CONDITIONDATAH_MSGENDAT);
	}

	params->IFOData.data[ifo_ind].data = NULL;
	LALCreateVector(status->statusPtr, &(params->IFOData.data[ifo_ind].data), params->NData);
	CHECKSTATUSPTR (status);
	dmro = params->IFOData.data[ifo_ind].data->data;

	/* get time info */
	params->IFOData.data[ifo_ind].epoch.gpsSeconds = inout->sequences[si].range.dTime.startSec;
	params->IFOData.data[ifo_ind].epoch.gpsNanoSeconds = inout->sequences[si].range.dTime.startNan;

	params->IFOData.data[ifo_ind].f0 = 0.0;
	params->IFOData.data[ifo_ind].deltaT = 1.0 / params->fs;

	/* tf-plane rectangle */
	params->cmint = (UINT4)floor((params->cmintR8 - (REAL8)(inout->sequences[si].range.dTime.startSec) - 1E-9 * (REAL8)(inout->sequences[si].range.dTime.startNan)) / params->T);
	params->cmaxt = (UINT4)ceil((params->cmaxtR8 - inout->sequences[si].range.dTime.startSec - 1E-9 * inout->sequences[si].range.dTime.startNan) / params->T);

	/* cast */
	switch(inout->sequences[si].type) {

	case real_4:
	  memcpy(dmro,inout->sequences[si].data.real4,params->NData*sizeof(REAL4));
	  break;

#define TFCCAST(TYPE__,PTR__) \
    case TYPE__ : \
      for(i=0; i<params->NData; i++) \
	dmro[i] = (REAL4)inout->sequences[si].data.PTR__[i]; \
      break

	  TFCCAST(boolean_lu,boolean);
	  TFCCAST(char_s,chars);
	  TFCCAST(char_u,charu);
	  TFCCAST(int_2s,int2s);
	  TFCCAST(int_2u,int2u);
	  TFCCAST(int_4s,int4s);
	  TFCCAST(int_4u,int4u);
	  TFCCAST(int_8s,int8s);
	  TFCCAST(int_8u,int8u);
	  TFCCAST(real_8,real8);

#undef TFCCAST

	default:
	  ABORT ( status, CONDITIONDATAH_EDTYP, CONDITIONDATAH_MSGEDTYP );
	}

      } /* if(IFODATA) */
      

      /***** Filters Data *****/
      else if(strstr(inout->sequences[si].comment, INPUTNAME_PPFILTERPOLES)) {

	UINT4 i, ifo_ind, NPoles;
	char *ifo_name;
	COHIFO IFO;

	/* which IFO ? */
	ifo_name = inout->sequences[si].comment + strlen(INPUTNAME_PPFILTERPOLES);
	if(strstr(ifo_name,"lho2k")) {
	  IFO = LHO2K;
	} else if(strstr(ifo_name,"lho4k")) {
	  IFO = LHO4K;
	} else if(strstr(ifo_name,"llo4k")) {
	  IFO = LLO4K;
	} else if(strstr(ifo_name,"virgo")) {
	  IFO = VIRGO;
	} else if(strstr(ifo_name,"tama")) {
	  IFO = TAMA;
	} else if(strstr(ifo_name,"geo")) {
	  IFO = GEO;
	} else {
	  ABORT ( status, CONDITIONDATAH_EIFO, CONDITIONDATAH_MSGEIFO );
	}

	for(ifo_ind=0; ifo_ind<params->NIFO; ifo_ind++) {
	  if(params->IFOIND[ifo_ind] == IFO) {break;}
	}

	if(ifo_ind == params->NIFO) {
	  ABORT ( status, CONDITIONDATAH_ENIFO, CONDITIONDATAH_MSGENIFO );
	}

	if(!(inout->sequences[si].dimensions[0])) {
	  ABORT (status, CONDITIONDATAH_EIFIL, CONDITIONDATAH_MSGEIFIL);
	}

	NPoles = inout->sequences[si].dimensions[0] / 2 - 1;

	Poles[ifo_ind] = NULL;
	LALZCreateVector(status->statusPtr, Poles + ifo_ind, NPoles);
	CHECKSTATUSPTR (status);

	if(inout->sequences[si].type != real_8) {
	  ABORT (status, CONDITIONDATAH_EIFIL, CONDITIONDATAH_MSGEIFIL);
	}
	for(i=0; i<NPoles; i++) {
	  Poles[ifo_ind]->data[i].re = inout->sequences[si].data.real8[2*i];
	  Poles[ifo_ind]->data[i].im = inout->sequences[si].data.real8[2*i + 1];
	}
	
	Gain[ifo_ind].re = inout->sequences[si].data.real8[2*NPoles-2];
	Gain[ifo_ind].im = inout->sequences[si].data.real8[2*NPoles-1];
      }

      else if(strstr(inout->sequences[si].comment, INPUTNAME_PPFILTERZEROS)) {

	UINT4 i, ifo_ind, NZeros;
	char *ifo_name;
	COHIFO IFO;

	/* which IFO ? */
	ifo_name = inout->sequences[si].comment + strlen(INPUTNAME_PPFILTERPOLES);
	if(strstr(ifo_name,"lho2k")) {
	  IFO = LHO2K;
	} else if(strstr(ifo_name,"lho4k")) {
	  IFO = LHO4K;
	} else if(strstr(ifo_name,"llo4k")) {
	  IFO = LLO4K;
	} else if(strstr(ifo_name,"virgo")) {
	  IFO = VIRGO;
	} else if(strstr(ifo_name,"tama")) {
	  IFO = TAMA;
	} else if(strstr(ifo_name,"geo")) {
	  IFO = GEO;
	} else {
	  ABORT ( status, CONDITIONDATAH_EIFO, CONDITIONDATAH_MSGEIFO );
	}

	for(ifo_ind=0; ifo_ind<params->NIFO; ifo_ind++) {
	  if(params->IFOIND[ifo_ind] == IFO) {break;}
	}

	if(ifo_ind == params->NIFO) {
	  ABORT ( status, CONDITIONDATAH_ENIFO, CONDITIONDATAH_MSGENIFO );
	}

	if(!(inout->sequences[si].dimensions[0])) {
	  ABORT (status, CONDITIONDATAH_EIFIL, CONDITIONDATAH_MSGEIFIL);
	}

	NZeros = inout->sequences[si].dimensions[0] / 2 - 1;

	Zeros[ifo_ind] = NULL;
	LALZCreateVector(status->statusPtr, Zeros + ifo_ind, NZeros);
	CHECKSTATUSPTR (status);

	if(inout->sequences[si].type != real_8) {
	  ABORT (status, CONDITIONDATAH_EIFIL, CONDITIONDATAH_MSGEIFIL);
	}
	for(i=0; i<NZeros; i++) {
	  Zeros[ifo_ind]->data[i].re = inout->sequences[si].data.real8[2*i];
	  Zeros[ifo_ind]->data[i].im = inout->sequences[si].data.real8[2*i + 1];
	}
	
	Gain[ifo_ind].re = inout->sequences[si].data.real8[2*NZeros-2];
	Gain[ifo_ind].im = inout->sequences[si].data.real8[2*NZeros-1];
      }

      else {
	ABORT ( status, CONDITIONDATAH_EINPUT, CONDITIONDATAH_MSGEINPUT );
      }


    } /* for(si.. */

    /* check we have everything */
    for(si=0; si<params->NIFO; si++) {

      if(params->IFOInfo.CMat[si][si] == 0.0) {
	ABORT ( status, CONDITIONDATAH_ECMAT, CONDITIONDATAH_MSGECMAT );
      }

      if(!(params->IFOData.data[si].data->data)) {
	ABORT ( status, CONDITIONDATAH_EDMRO, CONDITIONDATAH_MSGEDMRO );
      }
      if(!Zeros[si] || !Poles[si]) {
	if(params->IFOInfo.nPreProcessed > 0) {
	  ABORT ( status, CONDITIONDATAH_EDMRO, CONDITIONDATAH_MSGEDMRO );
	} else {
	  doFilters = 0;
	}
      }
    }

    /* create filters */
    if(doFilters) {
      for(si=0; si<params->NIFO; si++) {

	COMPLEX16ZPGFilter filterInput;
	
	params->IFOInfo.filters[si] = NULL;

	filterInput.deltaT = 1.0 / params->fs;
	filterInput.zeros = Zeros[si];
	filterInput.poles = Poles[si];
	filterInput.gain = Gain[si];
	
	LALCreateREAL8IIRFilter(status->statusPtr, params->IFOInfo.filters + si, &filterInput);
	CHECKSTATUSPTR (status);

	LALZDestroyVector(status->statusPtr, Poles + si);
	CHECKSTATUSPTR (status);
	LALZDestroyVector(status->statusPtr, Zeros + si);
	CHECKSTATUSPTR (status);
	
      }
    }

    LALFree(Gain);
    LALFree(Poles);
    LALFree(Zeros);


  } /* if(searchMaster) */


  

  DETATCHSTATUSPTR (status);
  RETURN (status);
}

#else

/**************************************************************/
/**************************************************************/
/* ConditionData function for efficient stand-alone operation */
/**************************************************************/
/**************************************************************/

/* read in the following files:
CMat.coh
SignalPlus.coh
SignalCross.coh
LHO2Kseed.coh
LHO2KP.coh : whitening filter poles npoles\nre\tim\n
LHO2KZ.coh
...

NOTE: no response function (e.g. arm length) yet!!!
*/

#include <lal/Random.h>
#include <lal/DetResponse.h>
#include <lal/DetectorSite.h>
#include <lal/BandPassTimeSeries.h>
#include <sys/types.h>
#include <time.h>

void
LALConditionData(
    LALStatus             *status,
    LALSearchInput        *inout,
    void                  *searchParams,
    LALMPIParams          *mpiParams
                )
{
  CohereSearchParams *params;

  INITSTATUS (status, "LALConditionData", CONDITIONDATAC);
  ATTATCHSTATUSPTR (status);

  params = (CohereSearchParams *) searchParams;

  ASSERT (searchParams, status, CONDITIONDATAH_ENULL, CONDITIONDATAH_MSGENULL);

  /* do nothing if searchMaster */
  if(!(params->searchMaster)) {

    char dname[64], fbuf[256], buf[1024];
    FILE *in;
    UINT4 k, si, slength, Offset;
    COMPLEX16Vector *Poles, *Zeros;
    COMPLEX16 Gain;
    INT4 seed;
    static RandomParams *noiseParams;
    COMPLEX16ZPGFilter filterInput;
    COMPLEX8ZPGFilter filterInputR4;
    REAL4IIRFilter *filter;
    REAL4Vector **SPlus, **SCross;
    REAL8 timeS, mean, p2c;
    REAL4 sigScale;

    LALDetAndSource dAs; /* for responses */
    LALSource source; /* for responses */
    LALDetAMResponse F;
    LIGOTimeGPS p_gps;
    LALDetector pDetector;

    /* plus 2 cross ratio */
    if(!(in=fopen("./Plus2Cross.coh","r"))) {
      ABORT ( status, SIGP, SIGPMSG );
    }

    fscanf(in,"%lg",&p2c);

    fclose(in);


    /* correlation matrix */
    if(!(in=fopen("./CMat.coh","r"))) {
      ABORT ( status, SIGP, SIGPMSG );
    }

    SPlus = (REAL4Vector **)LALMalloc(params->NIFO * sizeof(REAL4Vector *));
    SCross = (REAL4Vector **)LALMalloc(params->NIFO * sizeof(REAL4Vector *));

    for(si=0;si<params->NIFO;si++) {
      fgets(buf,1024,in);
      params->IFOInfo.CMat[si][0] = atof(strtok(buf,"\t\r\n"));
      for(k=1;k<params->NIFO;k++) {
	params->IFOInfo.CMat[si][k] = atof(strtok(NULL,"\t\r\n"));
      }
    }

    fclose(in);


    /* signals */
    /*
    seed = time(NULL);
    */

    /* read seed */
    sprintf(fbuf,"./RandomSeed.coh");
    if(!(in = fopen(fbuf,"r"))) {
      ABORT (status, NOSEED, NOSEEDMSG);
    }
    fscanf(in,"%i",&seed);
    fclose(in);
    

    if(!(in=fopen("./SignalPlus.coh","r"))) {
      if(!(in=fopen("./SignalPlusRandom.coh","r"))) {
	ABORT ( status, SIGP, SIGPMSG );
      } else {

	PassBandParamStruc lP, hP;
	REAL4TimeSeries R4Series;
	REAL4Vector *vect;
	char buf[256];
	UINT4 nn;

	fgets(buf,256,in);

	if(!strcmp(buf,"unique\n")) {
	  nn = 1;
	} else {
	  nn = params->NIFO;
	}

	fscanf(in,"%u",&slength);

	fscanf(in,"%i",&(lP.nMax));
	fscanf(in,"%lg",&(lP.f1));
	fscanf(in,"%lg",&(lP.a1));
	fscanf(in,"%lg",&(lP.f2));
	fscanf(in,"%lg",&(lP.a2));

	fscanf(in,"%i",&(hP.nMax));
	fscanf(in,"%lg",&(hP.f1));
	fscanf(in,"%lg",&(hP.a1));
	fscanf(in,"%lg",&(hP.f2));
	fscanf(in,"%lg",&(hP.a2));

	fclose(in);

	for(si=0; si<nn; si++) {

	SPlus[si] = NULL;
	LALCreateVector(status->statusPtr, SPlus + si, slength);
	CHECKSTATUSPTR (status);

	vect = NULL;
	LALCreateVector(status->statusPtr, &vect, 2*slength);
	CHECKSTATUSPTR (status);

	R4Series.deltaT = 1.0/16384.0;
	R4Series.data = vect;

	seed++;

	LALCreateRandomParams( status->statusPtr, &noiseParams, seed );
	CHECKSTATUSPTR (status);

	LALNormalDeviates(status->statusPtr, vect, noiseParams);
	CHECKSTATUSPTR (status);

	LALDestroyRandomParams( status->statusPtr, &noiseParams );
	CHECKSTATUSPTR (status);

	LALButterworthREAL4TimeSeries( status->statusPtr, &R4Series, &lP );
	CHECKSTATUSPTR (status);

	LALButterworthREAL4TimeSeries( status->statusPtr, &R4Series, &hP );
	CHECKSTATUSPTR (status);

	memcpy((SPlus[si])->data, vect->data + slength/2, slength * sizeof(REAL4));

	LALDestroyVector(status->statusPtr, &vect);
	CHECKSTATUSPTR (status);
	}

	/*
	for(si=1;si<slength;si++) {
	  printf("%g\n",(SPlus[0])->data[si]);
	}
	return;
	*/

	if(!strcmp(buf,"unique\n")) {
	  for(si=1;si<params->NIFO;si++) {

	    SPlus[si] = NULL;
	    LALCreateVector(status->statusPtr, SPlus + si, slength);
	    CHECKSTATUSPTR (status);

	    (SPlus[si])->length = slength;
	    memcpy((SPlus[si])->data, (SPlus[0])->data, slength * sizeof(REAL4));
	  }
	} 
      }
    } else {

      fscanf(in,"%u",&slength);

      SPlus[0] = NULL;
      LALCreateVector(status->statusPtr, SPlus, slength);
      CHECKSTATUSPTR (status);

      for(k=0;k<slength;k++) {
	fscanf(in,"%g",(SPlus[0])->data+k);
      }

      fclose(in);

      for(si=1;si<params->NIFO;si++) {

	SPlus[si] = NULL;
	LALCreateVector(status->statusPtr, SPlus + si, slength);
	CHECKSTATUSPTR (status);

	(SPlus[si])->length = slength;
	memcpy((SPlus[si])->data, (SPlus[0])->data, slength * sizeof(REAL4));
      }

    }

    if(!(in=fopen("./SignalCross.coh","r"))) {
      if(!(in=fopen("./SignalCrossRandom.coh","r"))) {
	ABORT ( status, SIGC, SIGCMSG );
      } else {

	PassBandParamStruc lP, hP;
	REAL4TimeSeries R4Series;
	REAL4Vector *vect;
	char buf[256];
	UINT4 nn;

	fgets(buf,256,in);

	if(!strcmp(buf,"unique\n")) {
	  nn = 1;
	} else {
	  nn = params->NIFO;
	}

	fscanf(in,"%u",&slength);

	fscanf(in,"%i",&(lP.nMax));
	fscanf(in,"%lg",&(lP.f1));
	fscanf(in,"%lg",&(lP.a1));
	fscanf(in,"%lg",&(lP.f2));
	fscanf(in,"%lg",&(lP.a2));

	fscanf(in,"%i",&(hP.nMax));
	fscanf(in,"%lg",&(hP.f1));
	fscanf(in,"%lg",&(hP.a1));
	fscanf(in,"%lg",&(hP.f2));
	fscanf(in,"%lg",&(hP.a2));

	fclose(in);

	for(si=0; si<nn; si++) {

	SCross[si] = NULL;
	LALCreateVector(status->statusPtr, SCross + si, slength);
	CHECKSTATUSPTR (status);

	vect = NULL;
	LALCreateVector(status->statusPtr, &vect, 2*slength);
	CHECKSTATUSPTR (status);

	R4Series.deltaT = 1.0/16384.0;
	R4Series.data = vect;

	seed++;

	LALCreateRandomParams( status->statusPtr, &noiseParams, seed );
	CHECKSTATUSPTR (status);

	LALNormalDeviates(status->statusPtr, vect, noiseParams);
	CHECKSTATUSPTR (status);

	LALDestroyRandomParams( status->statusPtr, &noiseParams );
	CHECKSTATUSPTR (status);

	LALButterworthREAL4TimeSeries( status->statusPtr, &R4Series, &lP );
	CHECKSTATUSPTR (status);

	LALButterworthREAL4TimeSeries( status->statusPtr, &R4Series, &hP );
	CHECKSTATUSPTR (status);

	memcpy((SCross[si])->data, vect->data + slength/2, slength * sizeof(REAL4));
	
	LALDestroyVector(status->statusPtr, &vect);
	CHECKSTATUSPTR (status);
	}

	if(!strcmp(buf,"unique\n")) {
	  for(si=1;si<params->NIFO;si++) {

	    SCross[si] = NULL;
	    LALCreateVector(status->statusPtr, SCross + si, slength);
	    CHECKSTATUSPTR (status);

	    (SCross[si])->length = slength;
	    memcpy((SCross[si])->data, (SCross[0])->data, slength * sizeof(REAL4));
	  }
	} 
      }
    } else {

      fscanf(in,"%u",&slength);

      SCross[0] = NULL;
      LALCreateVector(status->statusPtr, SCross, slength);
      CHECKSTATUSPTR (status);

      for(k=0;k<slength;k++) {
	fscanf(in,"%g",(SCross[0])->data+k);
      }

      fclose(in);

      for(si=1;si<params->NIFO;si++) {

	SCross[si] = NULL;
	LALCreateVector(status->statusPtr, SCross + si, slength);
	CHECKSTATUSPTR (status);

	(SCross[si])->length = slength;
	memcpy((SCross[si])->data, (SCross[0])->data, slength * sizeof(REAL4));
      }

    }

    /* do each IFO */
    for(si=0;si<params->NIFO;si++) {
      
      UINT4 i, NPoles, NZeros;

      switch(params->IFOIND[si]) {
      case LHO2K:
	sprintf(dname,"LHO2K");
	break;
      case LHO4K:
	sprintf(dname,"LHO4K");
	break;
      case LLO4K:
	sprintf(dname,"LLO4K");
	break;
      case VIRGO:
	sprintf(dname,"VIRGO");
	break;
      case GEO:
	sprintf(dname,"GEO");
	break;
      case TAMA:
	sprintf(dname,"TAMA");
	break;
      default:
	ABORT ( status, INITSEARCHH_EIFO, INITSEARCHH_MSGEIFO);
      }



      /***** Inverse Filters Data *****/
      sprintf(fbuf,"./%sP.coh",dname);
      if(!(in = fopen(fbuf,"r"))) {
	fprintf(stderr,"%s\n",dname);
	ABORT ( status, POLE, POLEMSG );
      }

      fscanf(in,"%u",&NPoles);

      Poles = NULL;
      LALZCreateVector(status->statusPtr, &Poles, NPoles);
      CHECKSTATUSPTR (status);

      for(i=0; i<NPoles; i++) {
	fscanf(in,"%lg%lg",&(Poles->data[i].re),&(Poles->data[i].im));
	}
	
      fscanf(in,"%lg%lg",&(Gain.re),&(Gain.im));

      fclose(in);

      
      sprintf(fbuf,"./%sZ.coh",dname);
      if(!(in = fopen(fbuf,"r"))) {
	fprintf(stderr,"%s\n",dname);
	ABORT ( status, ZERO, ZEROMSG );
      }

      fscanf(in,"%u",&NZeros);

      Zeros = NULL;
      LALZCreateVector(status->statusPtr, &Zeros, NZeros);
      CHECKSTATUSPTR (status);

      for(i=0; i<NZeros; i++) {
	fscanf(in,"%lg%lg",&(Zeros->data[i].re),&(Zeros->data[i].im));
	}
	
      fscanf(in,"%lg%lg",&(Gain.re),&(Gain.im));

      fclose(in);


      /* create direct filter */
      filter = NULL;
      
      filterInputR4.deltaT = 1.0 / params->fs;

      filterInputR4.zeros = filterInputR4.poles = NULL;
      LALCCreateVector(status->statusPtr, &(filterInputR4.zeros), NZeros);
      CHECKSTATUSPTR (status);
      LALCCreateVector(status->statusPtr, &(filterInputR4.poles), NPoles);
      CHECKSTATUSPTR (status);

      for(i=0;i<NZeros;i++) {
	filterInputR4.zeros->data[i].re = (REAL4)Poles->data[i].re;
	filterInputR4.zeros->data[i].im = (REAL4)Poles->data[i].im;
      }

      for(i=0;i<NPoles;i++) {
	filterInputR4.poles->data[i].re = (REAL4)Zeros->data[i].re;
	filterInputR4.poles->data[i].im = (REAL4)Zeros->data[i].im;
      }

      filterInputR4.gain.re = 1.0/Gain.re;
      filterInputR4.gain.im = 0.0;
	
      LALCreateREAL4IIRFilter(status->statusPtr, &filter, &filterInputR4);
      CHECKSTATUSPTR (status);

      LALCDestroyVector(status->statusPtr, &(filterInputR4.zeros));
      CHECKSTATUSPTR (status);

      LALCDestroyVector(status->statusPtr, &(filterInputR4.poles));
      CHECKSTATUSPTR (status);


      /* create whitening filter */
      params->IFOInfo.filters[si] = NULL;

      filterInput.deltaT = 1.0 / params->fs;
      filterInput.zeros = Zeros;
      filterInput.poles = Poles;
      filterInput.gain = Gain;
	
      LALCreateREAL8IIRFilter(status->statusPtr, params->IFOInfo.filters + si, &filterInput);
      CHECKSTATUSPTR (status);


      LALZDestroyVector(status->statusPtr, &Poles);
      CHECKSTATUSPTR (status);
      LALZDestroyVector(status->statusPtr, &Zeros);
      CHECKSTATUSPTR (status);



      /* read seed */
      sprintf(fbuf,"./%sseed.coh",dname);
      if(!(in = fopen(fbuf,"r"))) {
	fprintf(stderr,"%s\n",dname);
	ABORT (status, NOSEED, NOSEEDMSG);
      }
      fscanf(in,"%i",&seed);
      fscanf(in,"%g",&sigScale);
      fclose(in);


      /* IFO DATA */

      params->IFOData.data[si].data = NULL;
      LALCreateVector(status->statusPtr, &(params->IFOData.data[si].data), params->NData);
      CHECKSTATUSPTR (status);

      /* get time info */
      timeS = 700000000.0 - 0.5*((REAL8)params->NData)/params->fs;
      params->IFOData.data[si].epoch.gpsSeconds = (UINT4)floor(timeS);
      params->IFOData.data[si].epoch.gpsNanoSeconds = (UINT4)(1E9*(timeS-floor(timeS)));

      params->IFOData.data[si].f0 = 0.0;
      params->IFOData.data[si].deltaT = 1.0 / params->fs;

      /* tf-plane rectangle */
      params->cmint = (UINT4)floor((params->cmintR8 - (REAL8)(params->IFOData.data[si].epoch.gpsSeconds) - 1E-9 * (REAL8)(params->IFOData.data[si].epoch.gpsNanoSeconds)) / params->T);
      params->cmaxt = (UINT4)ceil((params->cmaxtR8 - params->IFOData.data[si].epoch.gpsSeconds - 1E-9 * params->IFOData.data[si].epoch.gpsNanoSeconds) / params->T);
      

      /* fill vector with random stuff & filter */
      LALCreateRandomParams( status->statusPtr, &noiseParams, seed );
      CHECKSTATUSPTR (status);

      LALNormalDeviates(status->statusPtr, params->IFOData.data[si].data, noiseParams);
      CHECKSTATUSPTR (status);

      LALIIRFilterREAL4Vector(status->statusPtr, params->IFOData.data[si].data, filter);
      CHECKSTATUSPTR (status);

      LALDestroyRandomParams( status->statusPtr, &noiseParams );
      CHECKSTATUSPTR (status);

      LALDestroyREAL4IIRFilter(status->statusPtr, &filter);
      CHECKSTATUSPTR (status);

      /* add signal with equal time delays & polarization angle = 0*/
      dAs.pSource = &source;
      source.equatorialCoords.longitude = 1.129814;
      source.equatorialCoords.latitude = 1.08161;
      source.equatorialCoords.system = COORDINATESYSTEM_EQUATORIAL;
      source.orientation = 0.0;
      p_gps.gpsSeconds = 700000000;
      p_gps.gpsNanoSeconds = 0;

      dAs.pDetector = &pDetector;
      switch(params->IFOIND[si]) {
      case LHO2K:
	pDetector = lalCachedDetectors[LALDetectorIndexLHODIFF];
	break;
      case LHO4K:
	pDetector = lalCachedDetectors[LALDetectorIndexLHODIFF];
	break;
      case LLO4K:
	pDetector = lalCachedDetectors[LALDetectorIndexLLODIFF];
	break;
      case VIRGO:
	pDetector = lalCachedDetectors[LALDetectorIndexGEO600DIFF];
	break;
      case GEO:
	pDetector = lalCachedDetectors[LALDetectorIndexGEO600DIFF];
	break;
      case TAMA:
	pDetector = lalCachedDetectors[LALDetectorIndexTAMA300DIFF];
	break;
      default:
	ABORT ( status, INITSEARCHH_EIFO, INITSEARCHH_MSGEIFO);
      }

      LALComputeDetAMResponse ( status->statusPtr, &F, &dAs, &p_gps );
      CHECKSTATUSPTR (status);

      Offset = params->NData / 2;

      F.plus *= sigScale;
      F.cross *= sigScale / p2c;

      for(k=0;k<(SPlus[0])->length;k++) {
	params->IFOData.data[si].data->data[k+Offset] += F.plus * (SPlus[si])->data[k];
      }

      for(k=0;k<(SCross[0])->length;k++) {
	params->IFOData.data[si].data->data[k+Offset] += F.cross * (SCross[si])->data[k];
      }


      mean = 0.0;
      for(k=0;k<params->NData;k++) {
	mean += params->IFOData.data[si].data->data[k];
      }

      mean /= (REAL4)params->NData;

      for(k=0;k<params->NData;k++) {
	params->IFOData.data[si].data->data[k] -= mean;
      }

    } /* loop over si */


    /* normalize data vectors */
    /*
    tPower = 0.0;
    for(i=0;i<params->NData;i++) {
      for(j=0;j<params->NIFO;j++) {
	tPower += params->IFOData.data[j].data->data[i] * params->IFOData.data[j].data->data[i];
      }
    }
    tPower = sqrt(tPower / (REAL8)(params->NData));
    for(i=0;i<params->NData;i++) {
      for(j=0;j<params->NIFO;j++) {
	params->IFOData.data[j].data->data[i] /= tPower;
      }
    }
    */

    /*
    {
      UINT4 i;
      for(i=0;i<params->NData;i++) {
	printf("%g\t%g\t%g\n",params->IFOData.data[0].data->data[i],params->IFOData.data[1].data->data[i],params->IFOData.data[2].data->data[i]);
      }
    }
      return;
    */
    for(si=0;si<params->NIFO;si++) {
    
      LALDestroyVector(status->statusPtr, SPlus + si);
      CHECKSTATUSPTR (status);

      LALDestroyVector(status->statusPtr, SCross + si);
      CHECKSTATUSPTR (status);

    }

    LALFree(SPlus);
    LALFree(SCross);
    
  } /* if(searchMaster) */


  DETATCHSTATUSPTR (status);
  RETURN (status);
}
#endif
