/******** <lalVerbatim file="FCTHierSupportCV"> ******** 
Authors: Edlund, Jeffrey; Fang, Hua; Tinto, Massimo 
$Id: FCTHierSupport.c,v 1.3 2003/06/23 18:54:28 jolien Exp $ 
********* </lalVerbatim> ********/ 

#include <lal/LALConstants.h>
#include <FCTGeneral.h>
#include <FCTHierSupport.h>

#include <stdio.h>
#include <lal/LALInspiralBank.h>
#include <lal/LALNoiseModels.h>

/* INT4 lalDebugLevel=0; where to put it? */

#define RESOLUTIONFAC  2 /* number of events across each dimension */
#define REFINEMENTFAC  2 /* reduction of event size in successive levels */


NRCSID( FCTHIERSUPPORTC, "$Id: FCTHierSupport.c,v 1.3 2003/06/23 18:54:28 jolien Exp $");

void FindThresholdAndUnits(LALStatus *status, FCTSearchParams *mySearchParams, multiDimData *NoiseData) {
  
  /* Parameters for LALInspiralCreateCoarseBank */
  InspiralTemplateList *plist=NULL;
  static InspiralCoarseBankIn coarseIn;
  static InspiralFineBankIn   fineIn;
  static INT4 pnlist,i;
  
  /* Transition */
  InspiralTemplateList      **list = &plist;
  INT4                      *nlist = &pnlist;
  
  /* What we are going to find */
  REAL8 ptau0,dtau0;
  REAL8 ptau3,dtau3;
  REAL8 dtau2,dtau4;
  REAL8 T_sun = 4.925e-6;
  REAL8 MMax, MMin, etaMin, etaMax;
  REAL8 f0;
  REAL8 A, B, C, D;
  
  static InspiralBankParams bankPars, bankParsOld;
  static InspiralTemplate tempPars;
  static InspiralMetric metric;
  static INT4 validPars, pass;
  static REAL8 a25, x01, x02, x11, x12, dist1, dist2, ndx1, ndx2;

  int j,k;  /* index variables */

  INITSTATUS( status, "FindThresholdAndUnits", FCTHIERSUPPORTC );
  ATTATCHSTATUSPTR(status);


  /* For now just set them by hand. */

  /* MM=0.90 */
  mySearchParams->hierLevelUnits[0].physicalStepSize[1] =  0.02985035;    /* dtau_0 */
  mySearchParams->hierLevelUnits[0].physicalStepSize[2] =  0.01654540;    /* dtau_2 */
  mySearchParams->hierLevelUnits[0].physicalStepSize[3] =  0.09036911;    /* dtau_3 */
  mySearchParams->hierLevelUnits[0].physicalStepSize[4] =  0.002511540;   /* dtau_4 */

  /* MM=0.99 */
  if ( mySearchParams->numHierLevels > 1 ) {
    mySearchParams->hierLevelUnits[1].physicalStepSize[1] =  0.009439511;
    mySearchParams->hierLevelUnits[1].physicalStepSize[2] =  0.005232114;
    mySearchParams->hierLevelUnits[1].physicalStepSize[3] =  0.02857722;
    mySearchParams->hierLevelUnits[1].physicalStepSize[4] =  0.0007942187;
  }

#ifdef FCT_COMPUTE_LAMBDA

  for(j = 0; j < mySearchParams->numHierLevels; j++){
#ifdef FCT_SO_DEBUG
    printf("In loop for Hierlevel = %i\n", j);
#endif

  /* Feed parameter coarseIn */
  coarseIn.mMin = mySearchParams->minMass;
  coarseIn.MMax = 2.0*mySearchParams->maxMass;
  coarseIn.mmCoarse = 0.99;  /* these two values need to be set for every Hier level */
  coarseIn.mmFine = 0.99;    /* always keep them the same */

  switch (j) {
  case 0:
    coarseIn.mmCoarse = 0.9;
    coarseIn.mmFine = 0.9;
  case 1:
    coarseIn.mmCoarse = 0.99;
    coarseIn.mmFine = 0.99;
    break;
  default:
    /* ? */
  }

  coarseIn.fLower = 40.;
  coarseIn.fUpper = 2000;
  coarseIn.iflso = 0;
  coarseIn.tSampling = 4096.;
  coarseIn.NoisePsd = LALLIGOIPsd;
  coarseIn.order = twoPN;
  coarseIn.space = Tau0Tau3;
  coarseIn.method = one;
  coarseIn.approximant = taylor;
  coarseIn.domain = TimeDomain;
  coarseIn.etamin = coarseIn.mMin * ( coarseIn.MMax - coarseIn.mMin) /
      pow(coarseIn.MMax,2.);
      
  
  ASSERT (coarseIn.NoisePsd, status, LALINSPIRALBANKH_ENULL, LALINSPIRALBANKH_MSGENULL);
  ASSERT (coarseIn.mMin > 0., status, LALINSPIRALBANKH_ESIZE, LALINSPIRALBANKH_MSGESIZE);
  ASSERT (coarseIn.MMax > 2.*coarseIn.mMin, status, LALINSPIRALBANKH_ESIZE, LALINSPIRALBANKH_MSGESIZE);
  ASSERT (coarseIn.mmCoarse > 0., status, LALINSPIRALBANKH_ESIZE, LALINSPIRALBANKH_MSGESIZE);
  ASSERT (coarseIn.fLower > 0., status, LALINSPIRALBANKH_ESIZE, LALINSPIRALBANKH_MSGESIZE);
  ASSERT (coarseIn.tSampling > 0., status, LALINSPIRALBANKH_ESIZE, LALINSPIRALBANKH_MSGESIZE);
  ASSERT (coarseIn.tSampling >= 2.*coarseIn.fUpper, status, LALINSPIRALBANKH_ESIZE, LALINSPIRALBANKH_MSGESIZE);
  ASSERT (coarseIn.space >= 0, status, LALINSPIRALBANKH_ECHOICE, LALINSPIRALBANKH_MSGECHOICE);
  ASSERT (coarseIn.space <= 1, status, LALINSPIRALBANKH_ECHOICE, LALINSPIRALBANKH_MSGECHOICE);
  
  MMax = coarseIn.MMax;
  MMin = 2*coarseIn.mMin;
  etaMax = 1.0/4.0;

  etaMin = coarseIn.mMin*(MMax-coarseIn.mMin)/pow(MMax,2);
  f0 = coarseIn.fLower;
      
  /* Number of templates is nlist */
  *nlist = 0;

  /* Set the elemets of the metric and tempPars structures in 
   conformity with the coarseIn structure */

  metric.NoisePsd = coarseIn.NoisePsd;
  metric.space = coarseIn.space;
  metric.iflso = coarseIn.iflso;
  LALInspiralSetParams(status->statusPtr, &tempPars, coarseIn);
  CHECKSTATUSPTR(status);

#ifdef FCT_SO_DEBUG
  printf("Did LALInspiralSetParams\n");
#endif

  /* Identify the boundary of search and parameters for the first lattice point */
  LALInspiralSetSearchLimits(status->statusPtr, &bankPars, coarseIn);
  
#ifdef FCT_SO_DEBUG
  printf("Did  LALInspiralSetSearchLimits\n");
#endif

  CHECKSTATUSPTR(status);
  tempPars.totalMass = coarseIn.MMax;
  tempPars.eta = 0.25;
  tempPars.fLower = coarseIn.fLower;
  tempPars.massChoice = totalMassAndEta; 
  LALInspiralParameterCalc(status->statusPtr, &tempPars);
  /*
  LALInspiralComputeParams(status->statusPtr, &tempPars, bankPars, coarseIn);
  CHECKSTATUSPTR(status);
  */

#ifdef FCT_SO_DEBUG
  printf("Did  LALInspiralParameterCalc\n");
#endif

  /* Compute the metric at this point, update bankPars and add it to the list */
  pass = 1;
  LALInspiralComputeMetric(status->statusPtr, &metric, tempPars, pass);
  CHECKSTATUSPTR(status);
#ifdef FCT_SO_DEBUG
  printf("Did  LALInspiralComputeMetric\n");
#endif

  pass = coarseIn.iflso;
  LALInspiralUpdateParams(status->statusPtr, &bankPars, metric, coarseIn.mmCoarse);
  CHECKSTATUSPTR(status);
#ifdef FCT_SO_DEBUG
  printf("Did  LALInspiralUpdateParams\n");
#endif  

  dtau0=bankPars.dx0;
  dtau3=bankPars.dx1;

  if (!(*list = (InspiralTemplateList*) 
	LALRealloc(*list, sizeof(InspiralTemplateList)*(*nlist+1)))) {
    ABORT(status, LALINSPIRALBANKH_EMEM, LALINSPIRALBANKH_MSGEMEM);
  }

  (*list)[*nlist].ID = *nlist; 
  (*list)[*nlist].params = tempPars; 
  (*list)[*nlist].metric = metric; ++(*nlist); 
  
  /* First lay templates along the equal mass curve; i.e. eta=1/4. 
     Choose the constant and the index converting the chirp times to
     one another along the curve depending on whether the templates
     are laid along the tau0-tau2 or tau0-tau3 parameters
  */

  switch (coarseIn.space) {
  case Tau0Tau2:
    ndx1 = 0.6;
    ndx2 = 1./ndx1;
    a25 = pow(64./5., ndx1)*(2435./8064.)/pow(LAL_PI*coarseIn.fLower,.4);
    break;
  case Tau0Tau3:
    a25 = LAL_PI_2 * pow(64./5., .4)/pow(LAL_PI * coarseIn.fLower, .6);
    ndx1 = 0.4;
    ndx2 = 2.5;
    break;
  }
  
  bankParsOld = bankPars;
  while (bankPars.x0 < bankPars.x0Max) {
#ifdef FCT_SO_DEBUG
    printf("In While loop: bankPars.x0 = %f,  bankPars.x0Max = %f\n",  bankPars.x0, bankPars.x0Max);
#endif  


    x01 = bankPars.x0 + bankPars.dx0;
    x11 = a25 * pow(x01,ndx1);
    x12 = bankPars.x1 + bankPars.dx1;
    x02 = pow(x12/a25,ndx2);
    dist1 = pow(bankPars.x0 - x01,2.) + pow(bankPars.x1 - x11, 2.);
    dist2 = pow(bankPars.x0 - x02,2.) + pow(bankPars.x1 - x12, 2.);
    if (dist1 < dist2) {
      bankPars.x0 = x01;
      bankPars.x1 = x11;
    } else {
      bankPars.x0 = x02;
      bankPars.x1 = x12;
    }
    /* If this is a valid point add it to our list */
    LALInspiralValidParams(status->statusPtr, &validPars, bankPars, coarseIn); 
    CHECKSTATUSPTR(status);
    if (validPars) {
      LALInspiralComputeParams(status->statusPtr, &tempPars, bankPars, coarseIn);
      CHECKSTATUSPTR(status);
      LALInspiralComputeMetric(status->statusPtr, &metric, tempPars, pass);
      CHECKSTATUSPTR(status);
      LALInspiralUpdateParams(status->statusPtr, &bankPars, metric, coarseIn.mmCoarse);
      CHECKSTATUSPTR(status);
      
      ptau3 = bankPars.dx1;
      if (dtau3 > ptau3) dtau3 = ptau3;
      
      /*  
	  On failure realloc() returns a NULL to outlist, hence there is
	  no need to explicitly free the outlist 
      */
      if (!(*list = (InspiralTemplateList*) 
            LALRealloc(*list, sizeof(InspiralTemplateList)*(*nlist+1)))) {
	ABORT(status, LALINSPIRALBANKH_EMEM, LALINSPIRALBANKH_MSGEMEM);
      }
      (*list)[*nlist].ID = *nlist; 
      (*list)[*nlist].params = tempPars; 
      (*list)[*nlist].metric = metric; 
      ++(*nlist); 
    }
  }
  
  /*   
  fprintf(stderr, "Number of templates along the eta=1/4 curve: %d\n\n",pnlist);
  fprintf(stderr, "Now we have dtau0 = %e\n",dtau0);
  fprintf(stderr, "            dtau3 = %e\n\n",dtau3);
  
  fprintf(stderr, "Using partial derivatives to calculate \n");
  fprintf(stderr, "the lower bound of dtau2 and dtau4...\n\n");
  */

  /* Calculate the coefficients for dtau2 */
  A = (1.0/756)*pow(f0,2.0/3)*pow(MMin*T_sun,2.0/3)*pow(LAL_PI,2.0/3)*(743+2772*etaMin);
  B = 5*(743-1386*etaMax)/(12096*pow(f0,1.0/3)*pow(MMax*T_sun,1.0/3)*pow(LAL_PI,4.0/3));
  dtau2 = A*dtau0 + B*dtau3;
  /*fprintf(stderr,"             dtau2 = %e\n",dtau2);*/
  
  C = pow(f0,4.0/3)*pow(MMax*T_sun,4.0/3)*pow(LAL_PI,4.0/3)*(1008*etaMin*(5429+12957*etaMin)-3058673)/1524096.0;
  D = 5*pow(f0,1.0/3)*pow(MMin*T_sun,1.0/3)*(3058673-252*etaMax*(5429+25914*etaMax))/pow(LAL_PI,2.0/3)/12192768.0;
  dtau4 = C*dtau0 + D*dtau3;
  /*fprintf(stderr,"             dtau4 = %e\n\n",dtau4);*/  

  mySearchParams->hierLevelUnits[j].physicalStepSize[1] = dtau0;
  mySearchParams->hierLevelUnits[j].physicalStepSize[2] = dtau2;
  mySearchParams->hierLevelUnits[j].physicalStepSize[3] = dtau3;
  mySearchParams->hierLevelUnits[j].physicalStepSize[4] = dtau4;

  }

#endif /* FCT_COMPUTE_LAMBDA */
  DETATCHSTATUSPTR(status);
  RETURN(status);
}

    
void MatchFCTUnitsToPhysicalUnits(LALStatus *status, FCTSearchParams *mySearchParams){
  /* This function needs to use the following information:
     mySearchParams->numHierLevels
     mySearchParams->fctInParams->dimension
     for(j = 0; j < mySearchParams->numHierLevels; j++) {
       mySearchParams->hierLevelUnits[j].range[0]         (length of time in Seconds)
       mySearchParams->hierLevelUnits[j].physicalStepSize[0] (time stepsize for FCT output in Seconds)
       mySearchParams->hierLevelUnits[j].freqCutOff
       mySearchParams->hierLevelUnits[j].indexRange[0]    (length of freq data after freq cutoff 
                                                           and length of time data after FCT)
       for(k = 0; k < mySearchParams->fctInParams->dimension; k++) {
         mySearchParams->hierLevelUnits[j].physicalStepSize[k]  (delta tau's)
       }
     }
     
     To generate: 
     for(j = 0; j < mySearchParams->numHierLevels; j++) {
       for(k = 0; k < mySearchParams->fctInParams->dimension; k++) {
         mySearchParams->hierLevelUnits[j].fctUnits[k]
         mySearchParams->hierLevelUnits[j].stride[k]
       }
     }
     
     The idea is that we use the stride in each dimension to match the
     physical stepsize as close as possible.  After stride is set, we
     should be able to convert from an index k to a physical value by:
     lambda[i] = k * mySearchParams->hierLevelUnits[j].fctUnits[k]
  */
  int j, k;
  
  INITSTATUS( status, "MatchFCTUnitsToPhysicalUnits", FCTHIERSUPPORTC );
  
  for(j = 0; j < mySearchParams->numHierLevels; j++) {
  
    switch ( mySearchParams->fctInParams->dimension) {
    case 5:
      /*Tau4 dimension */
      mySearchParams->hierLevelUnits[j].fctUnits[4] = 1.0/FCTGENERALH_F0/LambMax;
    case 4:
      /*Tau3 dimension */
      mySearchParams->hierLevelUnits[j].fctUnits[3] = 1.0/FCTGENERALH_F0/LambMax;
    case 3: 
      /*Tau2 dimension */
      mySearchParams->hierLevelUnits[j].fctUnits[2] =  1.0/FCTGENERALH_F0/LambMax;
    case 2:
      /*Tau0 dimension */
      mySearchParams->hierLevelUnits[j].fctUnits[1] = 5.0/3.0 / FCTGENERALH_F0 / LambMax;
      break;
    default:
      /* ABORT */
;
    }
    
    /*Tc dimension */
    mySearchParams->hierLevelUnits[j].fctUnits[0] = mySearchParams->hierLevelUnits[j].physicalStepSize[0];
    mySearchParams->hierLevelUnits[j].stride[0] = 1;

    for (k = 1; k < mySearchParams->fctInParams->dimension; k++){
      if (mySearchParams->hierLevelUnits[j].fctUnits[k] > mySearchParams->hierLevelUnits[j].physicalStepSize[k]) {
	/* ABORT */
      }
      /*      mySearchParams->hierLevelUnits[j].stride[k] = (INT4) (mySearchParams->hierLevelUnits[j].physicalStepSize[k] / mySearchParams->hierLevelUnits[j].fctUnits[k]);
      if ( mySearchParams->hierLevelUnits[j].stride[k] < 1 )
      
      mySearchParams->hierLevelUnits[j].stride[k] = 1;
      mySearchParams->hierLevelUnits[j].fctUnits[k] = mySearchParams->hierLevelUnits[j].stride[k] * mySearchParams->hierLevelUnits[j].fctUnits[k]; */
      mySearchParams->hierLevelUnits[j].stride[k] = (INT4)ceil(mySearchParams->hierLevelUnits[j].physicalStepSize[k]/mySearchParams->hierLevelUnits[j].fctUnits[k]);
    }
  }

  RETURN(status);
}


void
SetInitialEvents(LALStatus *status, FCTSearchParams *mySearchParams) {
  /* This function needs to use the following information:
     mySearchParams->numHierLevels
     mySearchParams->fctInParams->dimension
     mySearchParams->minMass
     mySearchParams->maxMass
     for(j = 0; j < mySearchParams->numHierLevels; j++) {
       mySearchParams->hierLevelUnits[j].range[0]         (length of time in Seconds)
       mySearchParams->hierLevelUnits[j].physicalStepSize[0] (time stepsize for FCT output in Seconds)
       mySearchParams->hierLevelUnits[j].freqCutOff
       mySearchParams->hierLevelUnits[j].indexRange[0]    (length of freq data after freq cutoff 
                                                           and length of time data after FCT)
       for(k = 0; k < mySearchParams->fctInParams->dimension; k++) {
         mySearchParams->hierLevelUnits[j].fctUnits[k]
       }
     }
     
     To generate:
     for(j = 0; j < mySearchParams->numHierLevels; j++) {
       for(k = 0; k < mySearchParams->fctInParams->dimension; k++) {
         mySearchParams->hierLevelUnits[j].range[k]
       }
     }  
     And create the linked list of events that cover the search volume:
  */
  UINT4 j, k; /* indecies over hierarchy level and dimension */
  UINT4 *i;   /* current location in units of stepsize */
  REAL4 *location, *minLocation, *maxLocation;
  FCTHierEventType head;
  FCTHierEventType *curEvent;

  INITSTATUS( status, "SetInitialEvents", FCTHIERSUPPORTC );
  ASSERT( mySearchParams, status,
	  FCTGENERALH_ENULL, FCTGENERALH_MSGENULL);
  ASSERT( mySearchParams->fctInParams, status,
	  FCTGENERALH_ENULL, FCTGENERALH_MSGENULL);
  ASSERT( mySearchParams->hierLevelEvents, status,
	  FCTGENERALH_ENULL, FCTGENERALH_MSGENULL);
  ASSERT( mySearchParams->hierLevelUnits, status,
	  FCTGENERALH_ENULL, FCTGENERALH_MSGENULL);
  for ( j = 0; j < mySearchParams->numHierLevels; j++) {
    ASSERT( mySearchParams->hierLevelUnits[j].range, status,
	    FCTGENERALH_ENULL, FCTGENERALH_MSGENULL);
  }

  /* Allocate for parameter coordinate vectors. */
  location = (REAL4*)
    LALMalloc(sizeof(REAL4) * (mySearchParams->fctInParams->dimension-1));
  if ( !location ) {
    ABORT( status, FCTGENERALH_EMEM, FCTGENERALH_MSGEMEM );
  }
  minLocation = (REAL4*)
    LALMalloc(sizeof(REAL4) * (mySearchParams->fctInParams->dimension-1)); 
  if ( !minLocation ) {
    LALFree( location );
    ABORT( status, FCTGENERALH_EMEM, FCTGENERALH_MSGEMEM );
  }
  maxLocation = (REAL4*)
    LALMalloc(sizeof(REAL4) * (mySearchParams->fctInParams->dimension-1)); 
  if ( !maxLocation ) {
    LALFree( location );
    LALFree( minLocation );
    ABORT( status, FCTGENERALH_EMEM, FCTGENERALH_MSGEMEM );
  }

  /* Find corners of hyperrectangle surrounding physical space. */
  ChirpTimes( minLocation, mySearchParams->fctInParams->dimension - 1,
	      mySearchParams->minMass, mySearchParams->minMass );
  memcpy( maxLocation, minLocation, sizeof(REAL4) *
	  (mySearchParams->fctInParams->dimension-1) );
  ChirpTimes( location, mySearchParams->fctInParams->dimension - 1,
	      mySearchParams->minMass, mySearchParams->maxMass );
  for ( k = 0; k < mySearchParams->fctInParams->dimension - 1; k++ ) {
    if ( location[k] < minLocation[k] )
      minLocation[k] = location[k];
    if ( location[k] > maxLocation[k] )
      maxLocation[k] = location[k];
  }
  ChirpTimes( location, mySearchParams->fctInParams->dimension - 1,
	      mySearchParams->maxMass, mySearchParams->maxMass );
  for ( k = 0; k < mySearchParams->fctInParams->dimension - 1; k++ ) {
    if ( location[k] < minLocation[k] )
      minLocation[k] = location[k];
    if ( location[k] > maxLocation[k] )
      maxLocation[k] = location[k];
  }


  /* DEBUG: reduce size of events by a factor of 10^(dimension-1)
  for ( k = 0; k < mySearchParams->fctInParams->dimension - 1; k++ ) {
    REAL4 diff = maxLocation[k] - minLocation[k];
    maxLocation[k] = minLocation[k] + 0.1*diff;
  } */



  /* Set event sizes for each hierarchy level. */
  for ( k = 1; k < mySearchParams->fctInParams->dimension; k++ ) {
    mySearchParams->hierLevelUnits[0].range[k] =
      ( maxLocation[k-1] - minLocation[k-1] )/(RESOLUTIONFAC);
  }
  for ( j = 1; j < mySearchParams->numHierLevels; j++ ) {
    for ( k = 1; k < mySearchParams->fctInParams->dimension; k++ ) {
      mySearchParams->hierLevelUnits[j].range[k] =
	mySearchParams->hierLevelUnits[j-1].range[k]/
	(REFINEMENTFAC);
    }
  }

  /* Free up some memory, and allocate an array of indecies. */
  LALFree( location );
  LALFree( maxLocation );
  i = (UINT4 *)LALMalloc( (mySearchParams->fctInParams->dimension-1) *
			  sizeof(UINT4) );
  if ( !i ) {
    LALFree( minLocation );
    ABORT( status, FCTGENERALH_EMEM, FCTGENERALH_MSGEMEM );
  }
  memset( i, 0, (mySearchParams->fctInParams->dimension-1)*sizeof(UINT4) );

  /* Initialize event lists. */
  memset( mySearchParams->hierLevelEvents, 0,
	  mySearchParams->numHierLevels*sizeof(FCTHierEventType *) );
  head.nextEvent = NULL;
  curEvent = &head;
  while ( i[mySearchParams->fctInParams->dimension - 2] < RESOLUTIONFAC ) {
    curEvent->nextEvent =
      createHierEvent( mySearchParams->fctInParams->dimension );
    curEvent = curEvent->nextEvent;
    if ( !curEvent ) {
      destroyEventList( head.nextEvent );
      LALFree( minLocation );
      LALFree( i );
      ABORT( status, FCTGENERALH_EMEM, FCTGENERALH_MSGEMEM );
    }
    for ( k = 0; k < mySearchParams->fctInParams->dimension-1; k++ )
      curEvent->location[k+1] = minLocation[k] +
	(i[k]+0.5)*mySearchParams->hierLevelUnits[0].range[k+1];
    i[0] += 1;
    for ( k = 0; k < mySearchParams->fctInParams->dimension - 2; k++ )
      if ( i[k] >= RESOLUTIONFAC ) {
	i[k] = 0;
	i[k+1] += 1;
      }
  }
  mySearchParams->hierLevelEvents[0] = head.nextEvent;


  mySearchParams->numHierLevelEvents[0] = 0;
  curEvent = mySearchParams->hierLevelEvents[0];
  while ( curEvent ) {
    mySearchParams->numHierLevelEvents[0]++;
    curEvent = curEvent->nextEvent;
  }

  /* Free up memory and exit. */
  LALFree( minLocation );
  LALFree( i );
  RETURN( status );
}


void LocateRange(LALStatus *status, FCTSearchParams *mySearchParams, REAL4 *minLocation, REAL4 *maxLocation) {
    

}

void FCTChunkAndLocateEvents(LALStatus *status, FCTSearchParams *mySearchParams) {
  /* This function should perform the FCT of the chunk as defined by mySearchParams->fctInParams.
     
     It should then locate events and add them to the linked list located in: 
       mySearchParams->hierLevelEvents[mySearchParams->curHierLevel + 1]
     When adding events please remember to increase:
       mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel + 1]
       
     You can look at the comments for the SetInitialEvents function above for one way to add events to the list.
     
     I suggest the following method to convert from FCT indices to a REAL4 physical location:
     event->location[i] = mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].fctUnits[i] * fctIndexLocation[i]
  */
}

/* void TranslateEventListAndBuildOutput (LALStatus *status, FCTSearchParams *mySearchParams) {
  
}*/

FCTHierEventType* createHierEvent(INT4 numDims) {
  FCTHierEventType *returnEvent; 
  
  returnEvent = LALMalloc(sizeof(FCTHierEventType));
  if (!returnEvent) {
    return(0);
  }
  returnEvent->location = LALMalloc(sizeof(INT4)*numDims);
  if (!returnEvent->location) {
    LALFree(returnEvent); 
    return(0);
  }
  returnEvent->nextEvent = 0;

  return(returnEvent);
}

void destroyEventList(FCTHierEventType* event) {
  FCTHierEventType *nextEvent; 
  
  while(event) {
    nextEvent = event->nextEvent; 
    LALFree(event->location);
    LALFree(event);
    event = nextEvent; 
  }

}


/* Inefficient functions computing chirp times (in seconds)
   from the binary masses (in solar masses). */

REAL4 tau0(REAL4 m1, REAL4 m2) {
  REAL4 mSec = m1 + m2;
  REAL4 eta = ( m1/mSec )*( m2/mSec );
  mSec /= LAL_MTSUN_SI;
  return ( (5.0/256.0)*mSec/eta )
    * pow( LAL_PI*mSec*FCTGENERALH_F0, -(8.0/3.0) );
}

REAL4 tau2(REAL4 m1, REAL4 m2) {
  REAL4 mSec = m1 + m2;
  REAL4 eta = ( m1/mSec )*( m2/mSec );
  mSec /= LAL_MTSUN_SI;
  return ( (5.0/192.0)*mSec/eta )
    * ( (11.0/4.0)*eta + (743.0/336.0) )
    * pow( LAL_PI*mSec*FCTGENERALH_F0, -2.0 );
}

REAL4 tau3(REAL4 m1, REAL4 m2) {
  REAL4 mSec = m1 + m2;
  REAL4 eta = ( m1/mSec )*( m2/mSec );
  mSec /= LAL_MTSUN_SI;
  return ( (3.0*LAL_PI/16.0)*mSec/eta )
    * pow( LAL_PI*mSec*FCTGENERALH_F0, -(5.0/3.0) );
}

REAL4 tau4(REAL4 m1, REAL4 m2) {
  REAL4 mSec = m1 + m2;
  REAL4 eta = ( m1/mSec )*( m2/mSec );
  mSec /= LAL_MTSUN_SI;
  return ( (15.0/128.0)*mSec/eta )
    * ( (617.0/144.0)*eta*eta + (5429.0/1008.0)*eta + (3058673.0/1016064.0) )
    * pow( LAL_PI*mSec*FCTGENERALH_F0, -(4.0/3.0) );
}

void ChirpTimes( REAL4 *tau, UINT4 dim, REAL4 m1, REAL4 m2 )
{
  UINT4 i;                             /* index over tau */
  REAL4 mSec = m1 + m2;                /* total system mass */
  REAL4 eta = ( m1/mSec )*( m2/mSec ); /* reduced mass ratio */
  mSec *= LAL_MTSUN_SI;                /* total mass in units of s */

  /* Compute tau0, tau2, tau3, tau4. */
  if ( dim > 0 )
    tau[0] = ( (5.0/256.0)*mSec/eta )
      * pow( LAL_PI*mSec*FCTGENERALH_F0, -(8.0/3.0) );
  if ( dim > 1 )
    tau[1] = ( (5.0/192.0)*mSec/eta )
      * ( (11.0/4.0)*eta + (743.0/336.0) )
      * pow( LAL_PI*mSec*FCTGENERALH_F0, -2.0 );
  if ( dim > 2 )
    tau[2] = ( (3.0*LAL_PI/16.0)*mSec/eta )
      * pow( LAL_PI*mSec*FCTGENERALH_F0, -(5.0/3.0) );
  if ( dim > 3 )
    tau[3] = ( (15.0/128.0)*mSec/eta )
      * ( (617.0/144.0)*eta*eta + (5429.0/1008.0)*eta +
	  (3058673.0/1016064.0) )
      * pow( LAL_PI*mSec*FCTGENERALH_F0, -(4.0/3.0) );

  /* Set any remaining fields to zero. */
  for ( i = 4; i < dim; i++ )
    tau[i] = 0.0;
  return;
}


void TranslateEventListAndBuildOutput (LALStatus       *status,
				       LALSearchOutput *output,
				       FCTSearchParams *mySearchParams) {
  FCTEventType       *eventList;  
  FCTEventType       *workingEvent;  
  
  INITSTATUS( status, "TranslateEventListAndBuildOutput", FCTHIERSUPPORTC); 
  ATTATCHSTATUSPTR (status);
  
  /*memset(eventList, 0, sizeof( FCTSearchParams ) );*/
  for (mySearchParams->curEventNum = 0;
       mySearchParams->curEventNum < mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel + 1];
       mySearchParams->curEventNum++){
    if (mySearchParams->curEventNum == 0) {
      mySearchParams->curEvent = mySearchParams->hierLevelEvents[mySearchParams->curHierLevel + 1];
      
      workingEvent = eventList = (FCTEventType*) LALMalloc(sizeof(FCTEventType));
      if (!workingEvent) {
	ABORT( status, HIERSUPPORTH_EVENT, HIERSUPPORTH_MSGEVENT );
      }
      memset(workingEvent, 0, sizeof(FCTEventType));
    } else {
      mySearchParams->curEvent = mySearchParams->curEvent->nextEvent;

      workingEvent->nextEvent = (FCTEventType*) LALMalloc(sizeof(FCTEventType));
      if (!workingEvent->nextEvent) {
	ABORT( status, HIERSUPPORTH_EVENT, HIERSUPPORTH_MSGEVENT );
      }
      workingEvent = workingEvent->nextEvent;
      memset(workingEvent, 0, sizeof(FCTEventType));
    }
    if (!workingEvent) {
      ABORT( status, HIERSUPPORTH_EVENT, HIERSUPPORTH_MSGEVENT );
    }
    workingEvent->time.gpsSeconds = mySearchParams->inputDataStartGPS.gpsSeconds;
    workingEvent->time.gpsNanoSeconds = mySearchParams->inputDataStartGPS.gpsNanoSeconds;
    workingEvent->effDist = 0;
    workingEvent->mass1 = 0;
    workingEvent->mass2 = 0;
    workingEvent->eta = 0;
    
    switch ( mySearchParams->fctInParams->dimension) {
    case 4:
      /*Tau3 dimension */
      workingEvent->t4 = mySearchParams->curEvent->location[4];
    case 3: 
      /*Tau2 dimension */
      workingEvent->t3 = mySearchParams->curEvent->location[3];
    case 2:
      /*Tau0 dimension */
      workingEvent->t2 = mySearchParams->curEvent->location[2];
      break;
    default:
      /* ABORT */
;
    }
    workingEvent->tC = mySearchParams->curEvent->location[0];
    workingEvent->t0 = mySearchParams->curEvent->location[1];

    workingEvent->snr = mySearchParams->curEvent->power;
    workingEvent->chisq = 0;
    workingEvent->sigma = 0;
  }
  LALBuildFCTOutput (status->statusPtr, output, 
		     mySearchParams->numHierLevelEvents[mySearchParams->curHierLevel + 1], eventList);

  while(eventList) {
    workingEvent = eventList->nextEvent; 
    LALFree(eventList);
    eventList = workingEvent; 
  }
  
  CHECKSTATUSPTR (status);
  
  DETATCHSTATUSPTR (status);        
  RETURN( status ); 
}
