 /******** <lalVerbatim file="FCCTonditionDataCV"> ******** 
Author: Wen, L. 
$Id: FCTConditionData.c,v 1.12 2002/04/22 18:38:39 lwen Exp $ 
********* </lalVerbatim> ********/ 

#include <FCTGeneral.h>
#include <FCTConditionData.h>

NRCSID( FCTCONDITIONDATAC, "$Id: FCTConditionData.c,v 1.12 2002/04/22 18:38:39 lwen Exp $");

void LALConditionData( 

		      LALStatus *status, 
		      LALSearchInput *inout, 
		      void *searchParams, 
		      LALMPIParams          *mpiParams
		      ) 
{ 
  
  FCTSearchParams *params;
  INT4                          i,j;
  INT4                          dmroIndex  = -1;
  INT4                          specIndex  = -1;
  INT4                          respIndex  = -1;
  REAL4                         *dummyI2    = NULL; 
  REAL4                        *dummyR4    = NULL;
  COMPLEX8                     *dummyC8    = NULL;
  REAL8                         distNorm = 0.0;
  INT4                          dataLength;
  
  /* DFT */
  COMPLEX8Vector* tmpIn; /*  time-series data  */
  ComplexFFTPlan* plan;
  FCTMomentType freqMoment;

  /* local variable*/
  /*REAL4 tempV=0; */
  INT4  respShape=1; /* check, 1 for U shape response, -1 for n shape response */
  REAL8 dt;
  REAL4 temp;  /* use if | R(f)|^2 is out of range */
  const REAL4   fac=1.0e-18;  /* check  */
  INT4  Nf0;

  INITSTATUS( status, "LALConditionData", FCTCONDITIONDATAC ); 
  ATTATCHSTATUSPTR (status);

#ifdef FCT_SO_DEBUG  
  /*     lalDebugLevel=24; */
#endif

  
  mpiParams=NULL;
  params = (FCTSearchParams *) searchParams;

#ifdef FCT_SO_DEBUG
   fprintf(stdout," in dataCondition()\n");
   fflush( stdout );
#endif

   /*  manipulating input to fit in FCT input */
   /* copy input in  */

   if ( params->searchMaster )
  {
    /* return immediately */
    DETATCHSTATUSPTR (status);
    RETURN (status);
  } 
   else /*  if slave, copy data from the ilwd input data format */
  {

#ifdef FCT_SO_DEBUG  
     fprintf(stdout,"inout->numberSequences=%i\n",inout->numberSequences); 
     fflush( stdout );
     fprintf(stdout,"inout->sequences[0].name=%s,INPUTNAME_IFODMRO=%s \n",inout->sequences[0].name,INPUTNAME_IFODMRO); 
     fflush( stdout );
    fprintf(stdout,"inout->sequences[1].name=%s\n",inout->sequences[1].name); 
     fflush( stdout );
    fprintf(stdout,"inout->sequences[2].name=%s\n",inout->sequences[2].name); 
     fflush( stdout );
     printf("strstr=%s\n",strstr(inout->sequences[0].name, "channel"));
#endif

    for ( i = 0; i < inout->numberSequences; ++i )
    {
      if (strstr( inout->sequences[i].name, INPUTNAME_IFODMRO))  
      {
        dmroIndex = i;

        dummyI2   = inout->sequences[i].data.real4;
      }
      else if (strstr( inout->sequences[i].name, INPUTNAME_SPECTRUM)) 
      {
        specIndex = i;
        dummyR4   = inout->sequences[i].data.real4;

      }
      else if (strstr( inout->sequences[i].name, INPUTNAME_RESPONSE)) 
      {
        respIndex = i;
        dummyC8   = inout->sequences[i].data.complex8;
      }
      else
        ABORT (status, CONDITIONDATA_EINPUT, CONDITIONDATA_MSGEINPUT); 
    }/* end for */

    if (dmroIndex == -1)
    {
      ABORT (status, CONDITIONDATA_EINPUT, CONDITIONDATA_MSGEINPUT); 
    }

    params->fctInParams->epoch.gpsSeconds =
      inout->sequences[dmroIndex].range.dTime.startSec;
    params->fctInParams->epoch.gpsNanoSeconds = 
      inout->sequences[dmroIndex].range.dTime.startNan;
    dt=params->fctInParams->dt=inout->sequences[dmroIndex].range.dTime.timeStepSize; /* make it local for now */
    FCTNyq=params->fctInParams->maxF=0.5/dt;
    
#ifdef FCT_SO_DEBUG  
    printf("epoch is %i,+%i, dt,maxF=%g, %g\n",params->fctInParams->epoch.gpsSeconds,params->fctInParams->epoch.gpsNanoSeconds,params->fctInParams->dt,params->fctInParams->maxF);
#endif
    dataLength=params->fctInParams->dataLength;

#ifdef FCT_SO_DEBUG
    printf(" specIndex,respIndex = %i,%i\n", specIndex,respIndex);
    printf(" data length is %i\n",dataLength);
    printf("input data length are %i,%i,%i\n",inout->sequences[dmroIndex].dimensions[0],inout->sequences[specIndex].dimensions[0],inout->sequences[respIndex].dimensions[0]);
#endif

    if ( inout->sequences[dmroIndex].dimensions[0] < dataLength || inout->sequences[specIndex].dimensions[0] < dataLength/2 || inout->sequences[respIndex].dimensions[0] < dataLength/2)
    {
      ABORT (status, CONDITIONDATA_EDATZ, CONDITIONDATA_MSGEDATZ); 
    }
    


    tmpIn=NULL;
    LALCCreateVector(status->statusPtr, &tmpIn, dataLength);
    CHECKSTATUSPTR (status);
    

    /*  do DFT here , a simple one */
      
    for (j = 0; j <dataLength; ++j)
    {
      tmpIn->data[j].re = (REAL4) dummyI2[j];
      tmpIn->data[j].im = 0.0;

#ifdef FCT_SO_DEBUG_DATA
      printf("data are %g\n",(REAL4) dummyI2[j]);
#endif

    }
    /* normalization s.t. h_bal=0 */
    LALFCTMoment(status->statusPtr, dataLength, tmpIn->data, &freqMoment);
    CHECKSTATUSPTR (status);
    
#ifdef FCT_SO_DEBUG
    printf("data mean, variance are %e, and %e\n",freqMoment.avg.re,freqMoment.sigma2.re);
#endif    
      
    /* normalize input to have h_bar=0, sigma_h=1 */

    for (j = 0; j <dataLength; ++j)
    {

      tmpIn->data[j].re -=freqMoment.avg.re;

      /*       tmpIn->data[j].re /=sqrt(freqMoment.sigma2.re); */
	
#ifdef FCT_SO_DEBUG_DATA
      printf("data are %g\n",(REAL4) tmpIn->data[j].re);
#endif

    }
    /*  Plan for doing the FFT on  the tseries */

    plan=NULL;
    /*  fwd FFT as in FFTW  with  -1 in the exponents */
    LALCreateForwardComplexFFTPlan(status->statusPtr, &plan, tmpIn->length,0);
    CHECKSTATUSPTR (status);

    /* necessary complex FFT? real FFT should be sufficient, check */
    LALCOMPLEX8VectorFFT(status->statusPtr, params->fctInParams->fctInput, tmpIn, plan);
    CHECKSTATUSPTR (status);
             
    params->fctInParams->dataLength/=2;
    params->fctInParams->fctInput->length/=2;
    dataLength=params->fctInParams->dataLength;

    /* partially initialized in InitSearch */
    params->fctInParams->distNorm *= (pow(dt/LAL_MTSUN_SI,1./6.)/dataLength);
    Nf0=dataLength*fCut/params->fctInParams->maxF >1 ? (INT4) (dataLength*fCut/params->fctInParams->maxF) :1;

#ifdef FCT_SO_DEBUG
    /* debug */
    printf("Nf0,distNorm2= %i,%e\n",Nf0,params->fctInParams->distNorm);
#endif
         
#ifdef FCT_SO_DEBUG_DATA
    for (j =0; j < dataLength*2; ++j)
    {
      printf(" FFT output  h(f)=%g+i %g\n", params->fctInParams->fctInput->data[j].re,params->fctInParams->fctInput->data[j].im);
    }
    
#endif

    /* check, dot-product to f^(-7/6), only (j/N)^(-7/6) and 
       1/N normalization, need it because FFTW are not normalized  */
    
    for (j =0; j <dataLength; ++j)
    {
	params->fctInParams->fctInput->data[j].re /= dataLength;
	params->fctInParams->fctInput->data[j].im /= dataLength;
    } 
    for (j =Nf0; j <dataLength; ++j)
    {
	params->fctInParams->fctInput->data[j].re /= pow((1.0*j)/dataLength,powInd);
	params->fctInParams->fctInput->data[j].im /=  pow((1.0*j)/dataLength,powInd); 
    }      
    

#ifdef FCT_SO_DEBUG
    LALFCTMoment(status->statusPtr, dataLength, params->fctInParams->fctInput->data, params->fctInParams->freqMoment);
    printf("After FFT,  input mean and variance are %e, %e,%e %e\n",params->fctInParams->freqMoment->avg.re,params->fctInParams->freqMoment->avg.im,params->fctInParams->freqMoment->sigma2.re,params->fctInParams->freqMoment->sigma2.im);
#endif

    /* incorporate the response and noise power in */
    
    /*
      [h_real(f)*resp (f)]_input * (conjugate)(Resp (f) ) ;  if resp is n shape, conjugate
    */
    
  if (specIndex !=0 && respIndex !=0)
  {
      
    for (j =0; j < dataLength; ++j)
    {
      tmpIn->data[j].re  =params->fctInParams->fctInput->data[j].re*dummyC8[j].re -respShape*params->fctInParams->fctInput->data[j].im*dummyC8[j].im;
      tmpIn->data[j].im  =params->fctInParams->fctInput->data[j].re*dummyC8[j].im+respShape*params->fctInParams->fctInput->data[j].im*dummyC8[j].re;
    } 
    
    
    /* deal with noise power which has a one-sided  PSD*/

    /*  
	If resp has U shape as in Teviet's code, 

	calculate h(f)/S(f)=
	([h(f)/ resp (f)]_input*resp (f)) /{[S_real(f)/|Resp|^2]_input * |Resp|^2} 
	       ={h_input *resp}_done /(S(f)*|resp|^2).
	       

      First, calculate S'(f)=(S(f) *|resp|^2)  
    */

    distNorm=0.0;  /* just to make sure */
    if (respShape ==1)
    {
    
      /* debug */
#ifdef FCT_SO_DEBUG
      printf("fac=%e, dummyR4=%e\n",fac,dummyR4[0]);
#endif

      for (j =0; j <dataLength; ++j)
      {
	dummyR4[j] /=fac; /* to avoid out of range at the next step */
	temp=dummyR4[j];
	dummyR4[j] =(temp * dummyC8[j].re) * dummyC8[j].re;
	dummyR4[j] +=((temp * dummyC8[j].im) * dummyC8[j].im);
      }

      /* calulate part of distNorm */

      /* Calculate sum (j/N0)^(-7/3)/(S(f)*||resp||^2) if resp is U shaped */

      for (j =Nf0; j <dataLength; ++j)
      {
	distNorm += ((pow((1.0*j)/dataLength,-2.*powInd)/dummyR4[j])/Mpc);
      }

#ifdef FCT_SO_DEBUG
      /* debug */
      printf("distNorm3= %e, %e\n",distNorm, params->fctInParams->distNorm);
#endif
     
      params->fctInParams->distNorm *= distNorm;
    }/* End if U shape  */
    else 
    {
      for (j =Nf0; j <dataLength; ++j)
      {
	distNorm += (((dummyC8[j].re/Mpc *dummyC8[j].re+dummyC8[j].im/Mpc *dummyC8[j].im)
		     *pow((1.0 *j)/dataLength,-2.*powInd))/dummyR4[j]);
      }
      
      
     params->fctInParams->distNorm *= distNorm;

    }/* endif n shape */

    
    /* debug */
#ifdef FCT_SO_DEBUG
    printf("distNorm= %e\n",params->fctInParams->distNorm);
#endif 

    /* Calculate h'(f)/S'(f) for resp with either shape */

    for (j =0; j <dataLength; ++j)
    {
      params->fctInParams->fctInput->data[j].re = tmpIn->data[j].re/dummyR4[j];
      params->fctInParams->fctInput->data[j].im = tmpIn->data[j].im/dummyR4[j];
    } 



#ifdef FCT_SO_DEBUG_DATA

    for (j=0; j<dataLength; j++)
    {
      printf("after resp/spec,  input h=%e+i %e, resp=%e+i%e, spect=%e\n", (params->fctInParams->fctInput->data[j].re),(params->fctInParams->fctInput->data[j].im), ((dummyC8[j].re)),((dummyC8[j].im)), dummyR4[j]);
      printf("fctInput are %g, %g\n",params->fctInParams->fctInput->data[j].re,params->fctInParams->fctInput->data[j].im);
    }
#endif


  }/* endif with noise spectrum and response */

#ifdef FCT_SO_DEBUG_DATA
    for (j =0; j < 2*dataLength; ++j)
    {
      printf(" input h(mid)(f)=%g+i %g\n", params->fctInParams->fctInput->data[j].re,params->fctInParams->fctInput->data[j].im);
    }
    
#endif

/* calculate the variance of the FCTInput and use it later at BuildEvent */

    LALFCTMoment(status->statusPtr, dataLength, params->fctInParams->fctInput->data, params->fctInParams->freqMoment);
    CHECKSTATUSPTR (status);
    
#ifdef FCT_SO_DEBUG
    printf("After resp/noise,  input mean and variance are %e, %e,%e %e\n",(params->fctInParams->freqMoment->avg.re),(params->fctInParams->freqMoment->avg.im),(params->fctInParams->freqMoment->sigma2.re),(params->fctInParams->freqMoment->sigma2.im));
#endif

    params->fctInParams->distNorm *= (sqrt(0.25/(dataLength-1)*(1./params->fctInParams->freqMoment->sigma2.re+1./params->fctInParams->freqMoment->sigma2.im)));

      /* debug */
#ifdef FCT_SO_DEBUG
    printf("distNorm3= %e\n",params->fctInParams->distNorm);
#endif


    LALCDestroyVector(status->statusPtr, &tmpIn);
    CHECKSTATUSPTR (status);
      
    LALDestroyComplexFFTPlan(status->statusPtr, &plan);
    CHECKSTATUSPTR (status);
    
  }/*  end if slave  */

#ifdef FCT_SO_DEBUG  
   fprintf(stdout," out of ConditionData()\n");
   fflush( stdout );
#endif

  DETATCHSTATUSPTR (status);
  RETURN( status ); 
} 


