/******** <lalVerbatim file="FCTInitSearchCV"> ******** 
Author: Wen, L. 
$Id: FCTInitSearch.c,v 1.11 2001/12/28 01:31:44 lwen Exp $ 
********* </lalVerbatim> ********/ 

#include <FCTGeneral.h>
#include <FCTInitSearch.h> 

NRCSID( FCTINITSEARCHC, "$Id: FCTInitSearch.c,v 1.11 2001/12/28 01:31:44 lwen Exp $");


void LALInitSearch( 
		   LALStatus           *status, 
		   void               **searchParams, 
		   LALInitSearchParams *initSearchParams 
		   ) 
{ 
  UINT4               j,dimension,dataLength;  /* ,errorFlag=0;*/
  FCTSearchParams   *params;
  INT4               *stride,*ofac;
  INT4   const       numParams=3;
  INT4   const            tCnt=6;

  /* FCT thing */
  LALFCTSetOversamplingFactorInput ofacTmp;

  /********************* LAL thing **********************************/ 
  INITSTATUS(status, "LALInitSearch", FCTINITSEARCHC ); /* mandatory for  LAL*/
  ATTATCHSTATUSPTR (status);   

#ifdef FCT_SO_DEBUG
  printf("entering initSearch()\n");
#endif  

#ifdef FCT_SO_ATTACH_DDD
  {
   unsigned int doze = 10;
   pid_t myPID;
   myPID = getpid( );

   fprintf(stdout, "pid %d sleeping for %d seconds\n", myPID, doze );
   fflush( stdout );
   
   sleep( doze );   
   fprintf(stdout,"pid %d awake\n", myPID );
   fflush( stdout );
   
  }
#endif
  

  if ( !(initSearchParams) )
  {
    ABORT( status, INITSEARCH_ENULL, INITSEARCH_MSGENULL);
  }

  
  /************************************************************ 
		       relevant FCT part 

		       Read in the search parameters
		       initialize FCT   

		       searchMaster= 1: searchmaster; 0:slave; -1: wrappermaster

  **************************************************************/

  *searchParams = params=LALMalloc (sizeof( FCTSearchParams ));  

  if ( ! params )
  {
    LALFree (*searchParams); *searchParams = NULL;
    ABORT (status, INITSEARCH_EALOC, INITSEARCH_MSGEALOC);
  }
  
  params->searchMaster=initSearchParams->nodeClass;
  params->firstTime= 1; 
  params->numSlaves =NULL;

  /*  check the validity of the input */
  if ( ! initSearchParams->argv )
  { 
    LALFree (*searchParams); *searchParams = NULL;
    ABORT( status, INITSEARCH_ENULL, INITSEARCH_MSGENULL ); 
  }


  if ( strcmp( initSearchParams->argv[0], "-filterparams" ) || 
       atoi(initSearchParams->argv[1]) <=0 ||
       atoi(initSearchParams->argv[2]) <=0 || 
       ((initSearchParams->argc -1-tCnt) != atoi(initSearchParams->argv[2])*numParams))
  { 
    LALFree (*searchParams); *searchParams = NULL;
    ABORT( status, INITSEARCH_EARGS, INITSEARCH_MSGEARGS ); 
  }

 
  
  dataLength=atoi(initSearchParams->argv[1]);
  dimension=atoi(initSearchParams->argv[2])+1; /* number including dim 0 */
  /* check dimension< numberPhasFunction */
   
  if (dimension >NUM_PHASE_FUNC+1)
  {
    /* need to check all deallocations */
    LALFree (*searchParams); *searchParams = NULL;
    ABORT( status, INITSEARCH_EARG2, INITSEARCH_MSGEARG2 ); 
  }     
  /*   
     params->startIndices      = LALMalloc(sizeof(INT4)*(dimension));
     params->endIndices        = LALMalloc(sizeof(INT4)*(dimension));
     params->indicesLength      = LALMalloc(sizeof(INT4)*(dimension));
     params->numSegPerDim         = LALMalloc(sizeof(INT4)*(dimension));
  */

#ifdef FCT_SO_DEBUG
  printf("check 1\n");
  printf("dim=%i \n",dimension);
#endif  

  params->segLengthPerRun        = LALMalloc(sizeof(INT4)*(dimension));
  if (!params->segLengthPerRun)
  {
    LALFree (*searchParams); *searchParams = NULL;
    ABORT( status, FCTGENERALH_EMEM, FCTGENERALH_MSGEMEM );
  } 
  stride             = LALMalloc(sizeof(INT4)*(dimension));
  if(!stride)
  {
    LALFree(params->segLengthPerRun);
    LALFree (*searchParams); *searchParams = NULL;
    ABORT( status, FCTGENERALH_EMEM, FCTGENERALH_MSGEMEM );
  }  
  ofac       = LALMalloc(sizeof(INT4)*(dimension));

  if(! ofac)
  {
    LALFree(params->segLengthPerRun);
    LALFree(stride);
    LALFree (*searchParams); *searchParams = NULL;
    ABORT( status, FCTGENERALH_EMEM, FCTGENERALH_MSGEMEM );
  } 

  /* check the memory allocation */

  params->minMass=atof(initSearchParams->argv[3]);
  params->maxMass=atof(initSearchParams->argv[4]);


#ifdef FCT_SO_DEBUG
  printf("check 2\n");
  printf("m1,m2=%f, %f\n",params->minMass,params->maxMass);
#endif  


     

  for (j=1; j<dimension; j++)
  {
    /* 
    params->startIndices[j] =atoi(initSearchParams->argv[3+numParams*(j-1)]);
    params->endIndices[j] =atoi(initSearchParams->argv[3+numParams*(j-1)+1]);
    params->indicesLength[j]=(params->endIndices[j]-params->startIndices[j]+1);
    */
    params->segLengthPerRun[j]=atoi(initSearchParams->argv[tCnt+numParams*(j-1)+1]);
    stride[j]=atoi(initSearchParams->argv[tCnt+numParams*(j-1)+2]);
    ofac[j]=atoi(initSearchParams->argv[tCnt+numParams*(j-1)+3]);
    /*     params->numSegPerDim[j]=params->indicesLength[j]/params->segLengthPerRun[j]; 
	   
	   totalNumSeg=totalNumSeg*params->numSegPerDim[j];
    */


    if (params->segLengthPerRun[j] <0 ||stride[j] <0 ||  
	params->minMass > params->maxMass || 
	stride[j] > params->segLengthPerRun[j])
    { 
      LALFree(params->segLengthPerRun);
      LALFree(stride);
      LALFree(ofac);
      ABORT( status, INITSEARCH_EARG2, INITSEARCH_MSGEARG2 ); 
    } 
  }

  params->totalNumSeg=0;
  params->dimension=dimension;
  

  /*  check that the dutyCycle >10, not yet  */


  params->segmentCompleted=0;

  params->hierLevelEvents = LALMalloc(sizeof(FCTHierEventType *));
  if (!params->hierLevelEvents)
  {
    LALFree(params->segLengthPerRun);
    LALFree(stride);
    LALFree(ofac);
    LALFree (*searchParams); *searchParams = NULL;
    ABORT (status, INITSEARCH_EALOC, INITSEARCH_MSGEALOC);
  } 


#ifdef FCT_SO_DEBUG
  printf("before SetInitialEvents\n");
  for (j=1; j<dimension; j++)
  {
    printf("segLength=%i\n",params->segLengthPerRun[j]);
  }
#endif  

  for (j=1; j<dimension; j++)
  {
    FCTBaseUnits[j-1] *=ofac[j];
  }
 /* call setInitEventList and set params->totalNumSeg */


  SetInitialEvents(status->statusPtr,params) ;
  CHECKSTATUSPTR (status);

#ifdef FCT_SO_DEBUG
  printf("after SetInitialEvents\n");
#endif  

  /* 
  fprintf( stdout, "\nInitial event list:\n" );
  {
    FCTHierEventType *curEvent = params->hierLevelEvents[0];
    while ( curEvent ) 
    {
      for ( j = 1; j < params->fctInParams->dimension; j++ )
      {  
	fprintf( stdout, " %5.2f", curEvent->location[j] );
      }  
      fprintf( stdout, "\n" );
      
      totalNumSeg++;
      curEvent = curEvent->nextEvent;
    }
  }
  */



  if (params->totalNumSeg==0)
  {
    LALFree(params->segLengthPerRun);
    LALFree(stride);
    LALFree(ofac);
    LALFree (*searchParams); *searchParams = NULL;
    ABORT( status, FCTGENERALH_ENULLE, FCTGENERALH_MSGENULLE );
  }
#ifdef FCT_SO_DEBUG
  fprintf( stdout, "# of events: %i\n",  params->totalNumSeg);
  fprintf (stdout,"totalNumSeg at j=%i are %i\n", j, params->totalNumSeg);
  fflush( stdout );
#endif





  /*  if slave  */


  if ( ! params->searchMaster )
  {
    

    FCTInStrucType    fctInStrus;
    LALCreateFCTPlanInput planIn;
    LALFCTSetUnitsInput setUnitsIn; 

    params->fctInParams=(FCTInParamType *) LALMalloc (sizeof(FCTInParamType)); 
    if ( !params->fctInParams)
    {
      LALFree(params->segLengthPerRun);
      LALFree(stride);
      LALFree(ofac);
      LALFree (*searchParams); *searchParams = NULL;
      ABORT (status, INITSEARCH_EALOC, INITSEARCH_MSGEALOC);
    }

  
      
    params->fctInParams->dataLength=dataLength;
    params->fctInParams->dimension=dimension;
  

    
    /* setting the threshold */
    params->fctInParams->ifoName=(CHAR *)LALMalloc( 2 * sizeof(CHAR) );
    strncpy(params->fctInParams->ifoName,initSearchParams->argv[5],2*sizeof(CHAR));
    params->fctInParams->prob=atof(initSearchParams->argv[6]);
    params->fctInParams->etaNorm=pow(2*LAL_PI/f0*LAL_PI/25./f0/f0,1./3.); 
    params->fctInParams->totalMassNorm=5./32./LAL_PI/LAL_PI/f0/LAL_MTSUN_SI;
    params->fctInParams->distNorm=LAL_MRSUN_SI*sqrt(5./96.)/pow(LAL_PI,2./3.)*(1.0+cos(incl)*cos(incl))*pow(2.,powInd);
    /* check with Teviet the factor of 2 */



    /* debug */
#ifdef FCT_SO_DEBUG
    printf("etaNorm=%e, MtotN=%e",params->fctInParams->etaNorm,params->fctInParams->totalMassNorm);
    /* debug */
    printf("distNorm= %e\n",params->fctInParams->distNorm);
    printf("ifoName=%s, prob=%e\n", params->fctInParams->ifoName,params->fctInParams->prob);
#endif  
    /*  Check:  things specific to FCT; from argv? */
    planIn.data_length=dataLength/2;
    planIn.number_of_dimensions=dimension; /*  including dim 0  */
    planIn.dimension_0_stride=1; 
    /*     planIn.ofac=1;  */ /* check, should come from input */ 

    for (j=0;j<dimension-1;j++)
    {
      planIn.phase_fn[j]=phaseFuncArray[j];
    }
    
    fctInStrus.planIn=planIn;

    /*  Define offset and as a constant there  */

    setUnitsIn.offset=0.0;
    setUnitsIn.delta=2.0/dataLength;

    fctInStrus.setUnitsIn=setUnitsIn;
    fctInStrus.maxFFTSeg=1;  /*  Check, as part of input? */
    params->fctInParams->cube=(LALFCTDataCube *)LALMalloc(sizeof(LALFCTDataCube));
    params->fctInParams->cube->stride[0]=planIn.dimension_0_stride;
    
    for (j=1; j<dimension; j++)
    {
      params->fctInParams->cube->stride[j]=stride[j];
      
      /* waiting for Philip's change */
      /*  
	  params->fctInParams->cube->ofac[j]=ofac[j];
      */
    }
    
    params->fctInParams->numDataCube=1; /* Check, for now */
    params->fctInParams->cube->start_locations[0]=0;/* due to some bizarre thing in fct.c */
    params->fctInParams->cube->end_locations[0]=dataLength/2;

#ifdef FCT_SO_DEBUG
    fprintf(stdout,"data_Length =%i\n",dataLength);
    fflush( stdout );
#endif
    

    params->fctInParams->freqMoment=(FCTMomentType*)LALMalloc(sizeof(FCTMomentType));
    params->fctInParams->ofac=ofac; 
    params->fctInParams->fctInput=NULL; /* necessary for LALCCreateVector() */
    LALCCreateVector(status->statusPtr, &(params->fctInParams->fctInput),dataLength);
    CHECKSTATUSPTR (status);
    
    params->fctInParams->fctPlan=NULL;
    LALMyFCTInitPlan(status->statusPtr, &fctInStrus, &(params->fctInParams->fctPlan));
    CHECKSTATUSPTR (status);
    
    if(!params->fctInParams->fctPlan)
    {
      LALFree(params->segLengthPerRun);
      LALFree(stride);
      LALFree(ofac);
      LALFree (*searchParams); *searchParams = NULL;
      ABORT( status, INITSEARCH_ENULL, INITSEARCH_MSGENULL);
    }
    
    for (j=1; j<dimension; j++)
    {
      ofacTmp.ofac=ofac[j];
      ofacTmp.dim=j;
      LALFCTSetOversamplingFactor(status->statusPtr,&ofacTmp,params->fctInParams->fctPlan);
      CHECKSTATUSPTR (status);
    }
    
    /* build initial linked list of physical regions to be searched */


  }/* endif slave  */
  else
  {
    LALFree(ofac);
    /* free Hier events list */
    destroyEventList(params->hierLevelEvents[0]);
#ifdef FCT_SO_DEBUG
    
    printf(" done freeing Hier event for the master\n");
#endif
  }

  /******************* LAL thing again ************/
  LALFree(stride);
  /* LALFree(ofac); */
#ifdef FCT_SO_DEBUG
  printf("leaving initSearch()\n");
#endif

  DETATCHSTATUSPTR (status);  
  RETURN( status ); 
} 


