/*----------------------------------------------------------------------- 
 * 
 * File Name: InitSearch.h
 *
 * Author: Brown, D. A.
 * 
 * Revision: $Id: InitSearch.h,v 1.22 2003/06/09 05:06:25 duncan Exp $
 * 
 *-----------------------------------------------------------------------
 */

#if 0
<lalVerbatim file="InitSearchHV">
Author: Brown, D. A.
$Id: InitSearch.h,v 1.22 2003/06/09 05:06:25 duncan Exp $
</lalVerbatim> 

<lalLaTeX>
\section{Header \texttt{InitSearch.h}}
\label{inspiral:InitSearch.h}

Provides {\tt LALInitSearch()} with a map between \texttt{-filterparams} and
search parameters, error messages and a function to read template parameters
from the command line.

\subsection*{Synopsis}
\begin{verbatim}
#include "InitSearch.h"
\end{verbatim}

This header should only be used by the \texttt{LALInitSearch()} function in
the \texttt{inspiral} shared object. It should not be used out of this context.
</lalLaTeX>
#endif

#ifndef _INITSEARCH_H
#define _INITSEARCH_H

#include "Inspiral.h"

#ifdef  __cplusplus
extern "C" {
#endif

NRCSID (INITSEARCHH, "$Id: InitSearch.h,v 1.22 2003/06/09 05:06:25 duncan Exp $");

#if 0
<lalLaTeX>
\subsection*{Command Line Arguments}
The number of arguments (including \texttt{-filterparams}) that the inspiral
shared object can take is defined in the header file. The number of arguments
is determined by the source of the template parameters. The ways to generate
the template bank are defined below.  In each case the number of command line
arguments should be exactly: 
\input{InspiralCommandLineArgsNUMC}
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsNUMC"> */
/* permissable numbers of command line arguments */
#define INSPARG_NBNK    16      /* generate templates using lal bank package */
#define INSPARG_NCMD    17      /* read templates from filterparams          */
#define INSPARG_NILD    15      /* read templates from InPut (ILWD)          */
/* </lalVerbatim> */

#if 0
<lalLaTeX>
This header also contains the map between position in the
\texttt{-filterparams} and the meaning of the command. The shared object
performs as strict as possible checking on the command input, but carse should
be taken to ensure that appropriate numbers should be given. Here we document
the meaning of each of the parameters and give typical values. The 
\verb|#define| gives the index of the \texttt{argv[]} array corresponding to to
a particular parameter.

</lalLaTeX>
#endif
/* command line arguments */
/* <lalVerbatim file="InspiralCommandLineArgsNPTS"> */
#define INSPARG_NPTS    1       /* number of points in a segment        */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsNPTS}
\noindent
Number of points in a \texttt{DataSegment}. The shared object takes an array
of time domain input data and splits it into one or more \texttt{DataSegment}
structures which are the basic unit of the \texttt{findchirp} package. More
details on the \texttt{DataSegment} segment structure can be found in the LAL
documentation for that package. This value must be greater than zero and a
power of two. Typical values are $262144$--$1048576$.
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsNSEG"> */
#define INSPARG_NSEG    2       /* number of data segments              */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsNSEG}
\noindent
Number of \texttt{DataSegment} structures to create.  This value must be
greater than zero and a power of two. Typical values are $1$--$16$.
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsOVRL"> */
#define INSPARG_OVRL    3       /* data segment overlap                 */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsOVRL}
\noindent 
The overlap between the time domain data in condsecutive \texttt{DataSegment}
structures.  This value must be $\ge 0$. Typical values are $0$ or half the
length of a data segment. In order to avoid filtering problems, it is
suggested at the moment to use half the number of points in a data segment.
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsFLOW"> */
#define INSPARG_FLOW    4       /* low frequency cutoff                 */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsFLOW}
\noindent 
The low frequency cutoff in Hz used by \texttt{findchirp}. It is suggested to
set this to $40.0$ for LIGO I. This value must be greater than zero.
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsTRNC"> */
#define INSPARG_TRNC    5       /* inverse spec trunc                   */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsTRNC}
\noindent 
The data conditioning function \texttt{FindChirpSPData()} has the ability to
truncate the inverse power spectrum in the time domain to remove high $Q$ 
features in the spectrum. A non-zero value will cause this trunctation to be
performed. \texttt{invSpecTrunc} specifies the number of non-zero points in the
truncated spectrum in the time domain. Typical values are $0$ for no
truncation of one quarter of the number of points in a data segment (e.g.
$262144/4 = 65536$).
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsDYNR"> */
#define INSPARG_DYNR    6       /* dynamic range                        */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsDYNR}
\noindent 
In order to store the strain power spectral density efficently it is necessary
to scale the values so that they are within the dynamic range of \texttt{REAL4}.
This parameters is $\log_2$ of the dynamic range scaling. Filter output is
independent of the parameter. A typical value for LIGO I data is $69.0$, so 
the response function is scaled by $2^{69} \sim 10^{20}$, hence the strain
power spectral density is scaled by $\sim 10^{40}$.
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsNCHI"> */
#define INSPARG_NCHI    7       /* number of chisq bins                 */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsNCHI}
\noindent 
Number of bins for the $\chi^2$ veto perfomed by \texttt{findchirp}. The must
be $\ge 0$. If set to zero, no $\chi^2$ veto is perfomed. Typical value is $8$.
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsMAXC"> */
#define INSPARG_MAXC    8       /* maxmimise snr over chirp length      */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsMAXC}
\noindent 
Maximise over chirp length or not. You decide.
</lalLaTeX>
#endif

/* <lalVerbatim file="InspiralCommandLineArgsRHOT"> */
#define INSPARG_RHOT    9       /* rhosq threshold                      */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsRHOT}
\noindent 
A list of two $\rho^2$ thresholds for the matched filter specified as
\verb|(1,2)| where \texttt{1} is the threshold for the coarse pass and
\texttt{2} is the threshold for the fine pass. Values must be positive and
chosen appropriately given the input data. If there is no fine bank, the fine
threshold is ignored.  If the filter output exceeds the threshold, a $\chi^2$
veto is performed, if requested. If not then fine templates are filtered or an
event generated as appropriate.
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsCHIT"> */
#define INSPARG_CHIT    10      /* chisq threshold                      */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsRHOT}
\noindent 
A list of two $\chi^2$ thresholds for the veto specified as \verb|(1,2)|
where \texttt{1} is the threshold for the coarse pass and \texttt{2} is the
threshold for the fine pass. Values must be positive and chosen appropriately
given the input data. If there is no fine bank, the fine threshold is ignored.
If the veto is below the threshold for the candidate event then fine templates
are filtered or an event generated as appropriate.
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsSPEC"> */
#define INSPARG_SPEC    11      /* psd estimation type                  */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsSPEC}
\noindent 
Controls how the power spectral density is computed:
\begin{itemize}
\item \textit{0} Read PSD from input data (traditional behaviour).
\item \textit{1} Compute PSD internally using mean method.
\item \textit{2} Compute PSD internally using meadian method.
\end{itemize}
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsNTXC"> */
#define INSPARG_NTXC    12      /* number of templates to exchange      */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsNTXC}
\noindent 
Number of templates to send to each slave per request. This is the number of
templates that the master will send to a slave when it requests templates to 
filter. We are currently determining the most optimum value for the parameter,
but the total number of templates divided by the number of slaves is a good 
place to start from.
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsCRVC"> */
#define INSPARG_CRVC    13      /* create rhosqVec (or other debugging) */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsCRVC}
\noindent 
This parameter is known as \texttt{create rhosqVec} for historical reasons,
but it now controls all the debugging and simulation performed by the shared
object. The possible values are:
\begin{itemize}
\item \texttt{0} No debugging or simulation output.
\item \texttt{1} Create a vector containing the filter output, $\rho^2(t)$,
for the last \texttt{DataSegment} filtered against the last template per call
to \texttt{FindChirpSlave()}. This is stored in the \texttt{multiDimData} sent
to the \texttt{wrapperAPI} and so can be written out as LIGO LW data.
\item \texttt{2} Tell \texttt{FindChirpSlave()} to perform a bank minimal
match simulation.
\end{itemize}
This should be set to $0$ unless debugging or simulation is required.
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsTSRC"> */
#define INSPARG_TSRC    14      /* template source                      */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsTSRC}
\noindent 
It is possible to obtain the template parameters in three ways. This parmeter
determines the method of generation and subsequent parameters in the
\texttt{-filterparams}. Possible values are:
\begin{itemize}
\item \texttt{0}: Generate by calls to the lal \texttt{bank} package. In this
  case the bank generation parameters must be specified on the command line,
  enclosed in parenthesis as the next and final argument (see below). In fact
  the inspiral shared object calls \texttt{LALFindChirpCreateInspiralBank()}
  which wraps the functions from \texttt{bank} so that the template bank is
  created in the appropriate linked list fashion for use by the search engine.
  
\item \texttt{1}: By reading template parameters from the
\texttt{-filterparams}. This is only suitable for small numbers of templates
$\le 10$ and is usuall used for testing and debugging. The number of templates
and a list should be specified.

\item \texttt{2}: By reading template parameters from the \texttt{InPut} data.
The method is currently only used for debugging. By creating a suitable ILWD
file which can be appended to the \texttt{wrapperAPI} input by the
\texttt{datacondAPI}, templates may be passed in along with the input data. 
No more parameters need to be specified in this case.
\end{itemize}
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsTPRM"> */
#define INSPARG_TPRM    15      /* template params                      */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsTPRM}
\noindent 
If generating templates using the \texttt{bank} package then this should
be a parenthesis enclosed list of the options passed to the bank generation
routines. \texttt{numLevel} should be $0$ or $1$ to generate a flat or a
heirarchical template bank respectively. For the definition of the other
options see the LAL documentation for the \texttt{bank} package:
\begin{verbatim}
(mMin, MMax, mmCoarse, mmFine, fLower, fUpper, tSampling, iflso, numLevel)
\end{verbatim}
  
If reading template parameters from the \texttt{-filterparams}. This should
be a single integer specifing the number of templates given on the command
line.
</lalLaTeX>
#endif
/* <lalVerbatim file="InspiralCommandLineArgsTMPL"> */
#define INSPARG_TMPL    16      /* templates from command line          */
/* </lalVerbatim> */
#if 0
<lalLaTeX>
\input{InspiralCommandLineArgsTPRM}
\noindent 
If reading template parameters from the \texttt{-filterparams}. This should
a parenthesis enclosed list of the template parameters mass 1 and mass 2. The
masses in each pair should be separated by commas and the pairs separated by
forward slashes. Masses should be given in uints of solar mass. For example:
\begin{verbatim}
(1.4,1.4/2.747,2.325/3.0,4.15)
\end{verbatim}
specifies the three templates $m_1 = 1.4, m_2 = 1.4$, 
$m_1 = 2.747, m_2 = 2.325$ and  $m_1 = 3.0, m_2 = 4.15$.
</lalLaTeX>
#endif

#if 0
<lalLaTeX> 
\subsection*{Error codes} 
</lalLaTeX>
#endif
/* <lalErrTable> */
#define INITSEARCHH_ENULL 1
#define INITSEARCHH_ENNUL 2
#define INITSEARCHH_EALOC 3
#define INITSEARCHH_EARGS 4
#define INITSEARCHH_ENUMZ 5
#define INITSEARCHH_ESEGZ 6
#define INITSEARCHH_EBINS 7
#define INITSEARCHH_EDTZO 8
#define INITSEARCHH_EOVLP 9
#define INITSEARCHH_ETRNC 10
#define INITSEARCHH_EFLOW 11
#define INITSEARCHH_EFREE 12
#define INITSEARCHH_ENLAL 13
#define INITSEARCHH_ENDAT 14
#define INITSEARCHH_ERHOT 15
#define INITSEARCHH_ECHIT 16
#define INITSEARCHH_EDYNR 17
#define INITSEARCHH_ETMPZ 18
#define INITSEARCHH_ETMPN 19
#define INITSEARCHH_ETSTR 20
#define INITSEARCHH_ENTXC 21
#define INITSEARCHH_ETSRC 22
#define INITSEARCHH_ETPRM 22
#define INITSEARCHH_EDFLG 23
#define INITSEARCHH_ESPEC 24
#define INITSEARCHH_EBANK 25
#define INITSEARCHH_MSGENULL "Null pointer"
#define INITSEARCHH_MSGENNUL "Non-null pointer"
#define INITSEARCHH_MSGEALOC "Memory allocation error"
#define INITSEARCHH_MSGEARGS "Wrong number of arguments"
#define INITSEARCHH_MSGENUMZ "Data segment length is zero or negative"
#define INITSEARCHH_MSGESEGZ "Number of data segments is zero or negative"
#define INITSEARCHH_MSGEBINS "Number of chi squared bins is zero or negative"
#define INITSEARCHH_MSGEDTZO "deltaT is zero or negative"
#define INITSEARCHH_MSGEOVLP "Segment overlap is negative"
#define INITSEARCHH_MSGETRNC "Duration of inverse spectrum in time domain is negative"
#define INITSEARCHH_MSGEFLOW "Inverse spectrum low frequency cutoff is negative"
#define INITSEARCHH_MSGEFREE "Memory free error"
#define INITSEARCHH_MSGENLAL "Tried to allocate to non-null pointer"
#define INITSEARCHH_MSGENDAT "No data read"
#define INITSEARCHH_MSGERHOT "Rhosq threshold error"
#define INITSEARCHH_MSGECHIT "Chisq threshold error"
#define INITSEARCHH_MSGEDYNR "Dynamic range exceeds range or REAL4"
#define INITSEARCHH_MSGETMPZ "Number of templates is incorrect"
#define INITSEARCHH_MSGETMPN "Templates list is null"
#define INITSEARCHH_MSGETSTR "Error parsing template string"
#define INITSEARCHH_MSGENTXC "Number of templates in coarse exchange is zero or negative"
#define INITSEARCHH_MSGETSRC "Invalid template source"
#define INITSEARCHH_MSGETPRM "Invalid template parameters"
#define INITSEARCHH_MSGEDFLG "Error parsing debug flag"
#define INITSEARCHH_MSGESPEC "Invalid method of PSD estimation"
#define INITSEARCHH_MSGEBANK "Internal bank generatation is depricated"
/* </lalErrTable> */

#if 0
\subsection*{Types}
#endif

void
InspiralTmpltsFromCmdLine (
    LALStatus                  *status,
    InspiralSearchParams       *params,
    CHAR                      **argv
    );

#if 0
<lalLaTeX>
\vfill{\footnotesize\input{InitSearchHV}}
\newpage\input{InitSearchC}
\newpage\input{InitSearchTmpltC}
</lalLaTeX> 
#endif

#ifdef  __cplusplus
}
#endif

#endif /* _INITSEARCH_H  */
